#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the FileUtils class."""

from __future__ import absolute_import
import sys
import unittest
from distutils import errors
from mock import patch, MagicMock as Mock

from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracFails, assertMockCalls, assertDiracSucceedsWith
from DIRAC import S_OK, S_ERROR
from six.moves import zip

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.FileUtils'

# pylint: disable=too-many-public-methods


class TestFileUtils(unittest.TestCase):
  """Test the different methods of the class."""

  def setUp(self):
    self.dm_mock = Mock()
    self.ops_mock = Mock()
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': self.dm_mock,
                       'DIRAC.ConfigurationSystem.Client.Helpers.Operations': self.ops_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()

  def tearDown(self):
    self.module_patcher.stop()

  def test_upload_tarball_missing(self):
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=False)) as exists_mock:
      assertDiracFails(upload('mypath', 'appTarTest'), self)
      exists_mock.assert_called_once_with('appTarTest')

  def test_upload_copy_fails(self):
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)) as exists_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('copy_testfail'))) as copy_mock:
      assertDiracFailsWith(upload('http://www.cern.ch/lcd-data/mypath', 'appTarTest'),
                            'could not copy because copy_testfail', self)
      exists_mock.assert_called_once_with('appTarTest')
      copy_mock.assert_called_once_with('appTarTest', '/afs/cern.ch/eng/clic/data/software/appTarTest')

  def test_upload_copy_works(self):
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)) as exists_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock()) as copy_mock:
      assertDiracSucceeds(upload('http://www.cern.ch/lcd-data/mypath', 'appTarTest'), self)
      exists_mock.assert_called_once_with('appTarTest')
      copy_mock.assert_called_once_with('appTarTest', '/afs/cern.ch/eng/clic/data/software/appTarTest')

  def test_upload_invalid_location(self):
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)):
      assertDiracFails(upload('http://www.mypath.com', 'appTarTest'), self)

  def test_upload_datman_upload_fails(self):
    self.dm_mock.DataManager().putAndRegister.return_value = S_ERROR('dataman_upload_testerr')
    self.ops_mock.Operations().getValue.return_value = '13984'
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)):
      assertDiracFailsWith(upload('/some/local/path', 'something/appTarTest.tgz'),
                            'dataman_upload_testerr', self)
      self.dm_mock.DataManager().putAndRegister.assert_called_once_with('/some/local/path/appTarTest.tgz',
                                                                         'something/appTarTest.tgz', '13984')
      self.ops_mock.Operations().getValue.assert_called_once_with('Software/BaseStorageElement', 'CERN-SRM')

  def test_upload_requestvalidation_fails(self):
    self.module_patcher.stop()
    req_mock = Mock()
    reqval_mock = Mock()
    reqval_mock.RequestValidator().validate.return_value = S_ERROR('validation_failed_testme')
    reqclient_mock = Mock()
    op_mock = Mock()
    op_list = [Mock(), Mock(), Mock(), Mock()]
    file_list = [Mock(), Mock(), Mock(), Mock()]
    copies_at_list = ['MyCopySE1', 'OtherSE', 'LastSE', '']
    op_mock.Operation.side_effect = op_list
    op_mock.File.side_effect = file_list
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': self.dm_mock,
                       'DIRAC.ConfigurationSystem.Client.Helpers.Operations': self.ops_mock,
                       'DIRAC.RequestManagementSystem.Client.Request': req_mock,
                       'DIRAC.RequestManagementSystem.private.RequestValidator': reqval_mock,
                       'DIRAC.RequestManagementSystem.Client.ReqClient': reqclient_mock,
                       'DIRAC.RequestManagementSystem.Client.Operation': op_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    self.dm_mock.DataManager().putAndRegister.return_value = S_OK()
    self.ops_mock.Operations().getValue.side_effect = [None, ['MyCopySE1', 'OtherSE', 'LastSE', '']]
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)):
      assertDiracFailsWith(upload('/some/local/path/', 'appTarTest.tgz'),
                            'validation_failed_testme', self)
      assertMockCalls(self.ops_mock.Operations().getValue,
                       [('Software/BaseStorageElement', 'CERN-SRM'), ('Software/CopiesAt', [])], self)
      request_to_test = req_mock.Request()
      assertEqualsImproved(request_to_test.RequestName, 'copy_appTarTest', self)
      assertEqualsImproved(request_to_test.SourceComponent, 'ReplicateILCSoft', self)
      assertMockCalls(request_to_test.addOperation, op_list, self)
      assertEqualsImproved(len(op_list), len(copies_at_list), self)
      index = 0
      for operation, se in zip(op_list, copies_at_list):
        assertEqualsImproved(operation.Type, 'ReplicateAndRegister', self)
        assertEqualsImproved(operation.TargetSE, se, self)
        operation.addFile.assert_called_once_with(file_list[index])
        assertEqualsImproved((file_list[index].LFN, file_list[index].GUID),
                              ('/some/local/path/appTarTest.tgz', ''), self)
        index += 1

  def test_upload_putrequest_fails(self):
    self.module_patcher.stop()
    req_mock = Mock()
    reqval_mock = Mock()
    reqval_mock.RequestValidator().validate.return_value = S_OK()
    reqclient_mock = Mock()
    reqclient_mock.ReqClient().putRequest.return_value = S_ERROR('ignore_test_err')
    op_mock = Mock()
    op_list = [Mock(), Mock(), Mock(), Mock()]
    file_list = [Mock(), Mock(), Mock(), Mock()]
    copies_at_list = ['MyCopySE1', 'OtherSE', 'LastSE', '']
    op_mock.Operation.side_effect = op_list
    op_mock.File.side_effect = file_list
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': self.dm_mock,
                       'DIRAC.ConfigurationSystem.Client.Helpers.Operations': self.ops_mock,
                       'DIRAC.RequestManagementSystem.Client.Request': req_mock,
                       'DIRAC.RequestManagementSystem.private.RequestValidator': reqval_mock,
                       'DIRAC.RequestManagementSystem.Client.ReqClient': reqclient_mock,
                       'DIRAC.RequestManagementSystem.Client.Operation': op_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    self.dm_mock.DataManager().putAndRegister.return_value = S_OK()
    self.ops_mock.Operations().getValue.side_effect = [None, ['MyCopySE1', 'OtherSE', 'LastSE', '']]
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)):
      assertDiracSucceedsWith(upload('/some/local/path/', 'appTarTest.tgz'), 'Application uploaded', self)
      assertMockCalls(self.ops_mock.Operations().getValue,
                       [('Software/BaseStorageElement', 'CERN-SRM'), ('Software/CopiesAt', [])], self)
      request_to_test = req_mock.Request()
      assertEqualsImproved(request_to_test.RequestName, 'copy_appTarTest', self)
      assertEqualsImproved(request_to_test.SourceComponent, 'ReplicateILCSoft', self)
      assertMockCalls(request_to_test.addOperation, op_list, self)
      assertEqualsImproved(len(op_list), len(copies_at_list), self)
      index = 0
      for operation, se in zip(op_list, copies_at_list):
        assertEqualsImproved(operation.Type, 'ReplicateAndRegister', self)
        assertEqualsImproved(operation.TargetSE, se, self)
        operation.addFile.assert_called_once_with(file_list[index])
        assertEqualsImproved((file_list[index].LFN, file_list[index].GUID),
                              ('/some/local/path/appTarTest.tgz', ''), self)
        index += 1

  def test_upload_putrequest_works(self):
    self.module_patcher.stop()
    req_mock = Mock()
    reqval_mock = Mock()
    reqval_mock.RequestValidator().validate.return_value = S_OK()
    reqclient_mock = Mock()
    reqclient_mock.ReqClient().putRequest.return_value = S_OK()
    op_mock = Mock()
    op_list = [Mock(), Mock(), Mock(), Mock()]
    file_list = [Mock(), Mock(), Mock(), Mock()]
    copies_at_list = ['MyCopySE1', 'OtherSE', 'LastSE', '']
    op_mock.Operation.side_effect = op_list
    op_mock.File.side_effect = file_list
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': self.dm_mock,
                       'DIRAC.ConfigurationSystem.Client.Helpers.Operations': self.ops_mock,
                       'DIRAC.RequestManagementSystem.Client.Request': req_mock,
                       'DIRAC.RequestManagementSystem.private.RequestValidator': reqval_mock,
                       'DIRAC.RequestManagementSystem.Client.ReqClient': reqclient_mock,
                       'DIRAC.RequestManagementSystem.Client.Operation': op_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    self.dm_mock.DataManager().putAndRegister.return_value = S_OK()
    self.ops_mock.Operations().getValue.side_effect = [None, ['MyCopySE1', 'OtherSE', 'LastSE', '']]
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)):
      assertDiracSucceedsWith(upload('/some/local/path/', 'appTarTest.tgz'), 'Application uploaded', self)
      assertMockCalls(self.ops_mock.Operations().getValue,
                       [('Software/BaseStorageElement', 'CERN-SRM'), ('Software/CopiesAt', [])], self)
      request_to_test = req_mock.Request()
      assertEqualsImproved(request_to_test.RequestName, 'copy_appTarTest', self)
      assertEqualsImproved(request_to_test.SourceComponent, 'ReplicateILCSoft', self)
      assertMockCalls(request_to_test.addOperation, op_list, self)
      assertEqualsImproved(len(op_list), len(copies_at_list), self)
      index = 0
      for operation, se in zip(op_list, copies_at_list):
        assertEqualsImproved(operation.Type, 'ReplicateAndRegister', self)
        assertEqualsImproved(operation.TargetSE, se, self)
        operation.addFile.assert_called_once_with(file_list[index])
        assertEqualsImproved((file_list[index].LFN, file_list[index].GUID),
                              ('/some/local/path/appTarTest.tgz', ''), self)
        index += 1

  def test_fullcopy_getallfiles(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.LOG' % MODULE_NAME, new=Mock()) as log_mock:
      assertDiracSucceeds(fullCopy('/my/src/directory/', '/my/destination/dir', '   ./__!@#.~/  '), self)
      log_mock.error.assert_called_once_with('You try to get all files, that cannot happen')

  def test_fullcopy_noitemsfound(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=None)) as glob_mock:
      assertDiracFailsWith(fullCopy('/my/src/directory/', '/my/destination/dir', './myfile123.txt  '),
                            'no items found', self)
      glob_mock.assert_called_once_with('/my/src/directory/myfile123.txt')

  def test_fullcopy_distutils_fails(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['/my/src/directory/file_1_globbed.log', '/my/src/directory/other__file.stdhep', 'lastfile_in_list.txt'])), \
         patch('%s.dir_util.create_tree' % MODULE_NAME, new=Mock(side_effect=errors.DistutilsFileError('test_dist_error'))):
      assertDiracFailsWith(fullCopy('/my/src/directory/', '/my/destination/dir', './myfile123.txt  '),
                            'test_dist_error', self)

  def test_fullcopy_copy1_fails(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['/my/src/directory/file_1_globbed.log', '/my/src/directory/other__file.stdhep', 'lastfile_in_list.txt'])), \
         patch('%s.dir_util.create_tree' % MODULE_NAME, new=Mock()) as createtree_mock, \
         patch('%s.os.path.isfile' % MODULE_NAME, new=Mock(return_value=True)) as isfile_mock, \
         patch('%s.shutil.copy2' % MODULE_NAME, new=Mock(side_effect=(None, None, EnvironmentError('testme_copy_enverr')))) as copy_mock:
      assertDiracFailsWith(fullCopy('/my/src/directory/', '/my/destination/dir', './myfile123.txt  '),
                            'testme_copy_enverr', self)
      assertMockCalls(createtree_mock, [('/my/destination/dir', ['file_1_globbed.log']),
                                          ('/my/destination/dir', ['other__file.stdhep']),
                                          ('/my/destination/dir', ['lastfile_in_list.txt'])], self)
      assertMockCalls(isfile_mock, ['/my/src/directory/file_1_globbed.log',
                                      '/my/src/directory/other__file.stdhep',
                                      '/my/src/directory/lastfile_in_list.txt'], self)
      assertMockCalls(copy_mock, [
          ('/my/src/directory/file_1_globbed.log', '/my/destination/dir/file_1_globbed.log'),
          ('/my/src/directory/other__file.stdhep', '/my/destination/dir/other__file.stdhep'),
          ('/my/src/directory/lastfile_in_list.txt', '/my/destination/dir/lastfile_in_list.txt')], self)

  def test_fullcopy_copy2_fails(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['/my/src/directory/file_1_globbed.log', '/my/src/directory/other__file.stdhep', 'lastfile_in_list.txt'])), \
         patch('%s.dir_util.create_tree' % MODULE_NAME, new=Mock()) as createtree_mock, \
         patch('%s.os.path.isfile' % MODULE_NAME, new=Mock(side_effect=[True, False])) as isfile_mock, \
         patch('%s.shutil.copy2' % MODULE_NAME, new=Mock()) as copy_mock, \
         patch('%s.shutil.copytree' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('tree_test_copy_error'))) as copytree_mock:
      assertDiracFailsWith(fullCopy('/my/src/directory/', '/my/destination/dir', './myfile123.txt  '),
                            'tree_test_copy_error', self)
      assertMockCalls(createtree_mock, [('/my/destination/dir', ['file_1_globbed.log']),
                                          ('/my/destination/dir', ['other__file.stdhep'])], self)
      assertMockCalls(isfile_mock, ['/my/src/directory/file_1_globbed.log',
                                      '/my/src/directory/other__file.stdhep'], self)
      copy_mock.assert_called_once_with(
          '/my/src/directory/file_1_globbed.log',
          '/my/destination/dir/file_1_globbed.log')
      copytree_mock.assert_called_once_with(
          '/my/src/directory/other__file.stdhep',
          '/my/destination/dir/other__file.stdhep')

  def test_fullcopy(self):
    from ILCDIRAC.Core.Utilities.FileUtils import fullCopy
    with patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['/my/src/directory/file_1_globbed.log', '/my/src/directory/other__file.stdhep', 'lastfile_in_list.txt'])), \
         patch('%s.dir_util.create_tree' % MODULE_NAME, new=Mock()), \
         patch('%s.os.path.isfile' % MODULE_NAME, new=Mock(side_effect=[True, False, True])), \
         patch('%s.shutil.copy2' % MODULE_NAME, new=Mock()), \
         patch('%s.shutil.copytree' % MODULE_NAME, new=Mock()):
      assertDiracSucceeds(fullCopy('/my/src/directory/', '/my/destination/dir', './myfile123.txt  '), self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Shared functionality for :mod:`~ILCDIRAC.Workflow.Modules.RootMacroAnalysis` and :mod:`~ILCDIRAC.Workflow.Modules.RootExecutableAnalysis`"""

from __future__ import absolute_import
import os

from DIRAC import S_OK, S_ERROR


class RootMixin(object):
  """Mixin class for :mod:`~ILCDIRAC.Workflow.Modules.RootMacroAnalysis` and :mod:`~ILCDIRAC.Workflow.Modules.RootExecutableAnalysis`"""

  def getRootEnvScript(self, _platform, _appname, _appversion):
    """create the environment script if it is not already available.

    Need to set LD_LIBRARY_PATH and PATH based on ROOTSYS

    As this is only called when we are not CVMFS native the ROOTSYS must have
    been set by :func:`~ILCDIRAC.Core.Utilities.TARsoft.configureRoot`. Function
    signature must conform to
    :func:`~ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation.getEnvironmentScript`,
    but none of the arguments are used.

    :param str _platform: Unused, Software platform
    :param str _appname: Unused, application name
    :param str _appversion: Unused, application version
    :returns: S_OK( pathToScript )
    """
    if 'ROOTSYS' not in os.environ:
      self.log.error("ROOTSYS is not set")
      return S_ERROR("ROOTSYS is not set")
    self.log.info("Creating RootEnv.sh with ROOTSYS: %s " % os.environ['ROOTSYS'])

    scriptName = "rootEnv.sh"
    with open(scriptName, "w") as script:
      if 'LD_LIBRARY_PATH' in os.environ:
        script.write('declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:$LD_LIBRARY_PATH\n')
      else:
        script.write('declare -x LD_LIBRARY_PATH=$ROOTSYS/lib\n')
        script.write('declare -x PATH=$ROOTSYS/bin:$PATH\n')
    return S_OK(os.path.join(os.getcwd(), scriptName))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Module that gets a file from its SRM definition.

:since: Aug 27, 2010
:author: sposs
"""

from __future__ import absolute_import
import os
import tempfile
import time

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Resources.Storage.StorageElement import StorageElementItem as StorageElement
from DIRAC.WorkloadManagementSystem.Client.JobMonitoringClient import JobMonitoringClient

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class GetSRMFile(ModuleBase):
  """When a file is not in the FileCatalog, it can still be obtained using this.

  and specifying the srm path.
  """

  def __init__(self):
    """Module initialization."""
    super(GetSRMFile, self).__init__()
    self.version = __RCSID__
    self.srmfiles = []
    self.files = []
    self.counter = 1

  def applicationSpecificInputs(self):
    """Resolve the srm files to get."""
    if not self.srmfiles:
      return S_ERROR("List of files to treat is not set")
    self.files = self.srmfiles
    return S_OK()

  def execute(self):
    """Run this."""
    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('Workflow status is not OK')
    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error("Failed to resolve input parameters:", result["Message"])
      return result
    if not self.srmfiles:
      LOG.error('Files txt was not found correctly: %s' % self.srmfiles)
      return S_ERROR('Files txt was not found correctly')

    if not isinstance(self.files[0], dict):
      LOG.error('Files were not found correctly: %s' % self.files)
      return S_ERROR('Files were not found correctly')

    # Now need to check that there are not that many concurrent jobs getting the overlay at the same time
    max_concurrent_running = self.ops.getValue('/GetSRM/MaxConcurrentRunning', 100)
    error_count = 0
    while True:
      if error_count > 10:
        LOG.error('JobDB Content does not return expected dictionary')
        return S_ERROR('Failed to get number of concurrent overlay jobs')
      jobMonitor = JobMonitoringClient()
      res = jobMonitor.getCurrentJobCounters({'ApplicationStatus': 'Downloading SRM files'})
      if not res['OK']:
        error_count += 1
        time.sleep(60)
        continue
      running = 0
      if 'Running' in res['Value']:
        running = res['Value']['Running']
      if running < max_concurrent_running:
        break
      else:
        time.sleep(60)

    self.setApplicationStatus('Downloading SRM files')
    for filed in self.files:
      if 'file' not in filed or 'site' not in filed:
        LOG.error('Dictionnary does not contain correct keys')
        return S_ERROR('Dictionnary does not contain correct keys')
      start = os.getcwd()
      downloadDir = tempfile.mkdtemp(prefix='InputData_%s' % (self.counter), dir=start)
      os.chdir(downloadDir)
      storageElement = StorageElement(filed['site'])
      result = storageElement.getFile(filed['file'])
      if result['Value']['Failed']:
        result = storageElement.getFile(filed['file'])
      os.chdir(start)
      if result['Value']['Failed']:
        LOG.error("Failed to get the file from storage:", result['Value']['Failed'])
        return result
      self.counter += 1

    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Several utilities to "guess" the files names and the paths.

:author: sposs
:since: March 13th, 2013
"""
from __future__ import absolute_import
import six
__RCSID__ = "$Id$"

import os
from DIRAC import S_OK, S_ERROR
from DIRAC import gLogger
from ILCDIRAC.Core.Utilities.FilenameEncoder import FilenameEncoder, decodeFilename
###############################################################################


def getProdFilenameFromInput(inputfile, outfileOriginal, prodID, jobID):
  """Build the output file names based on inputfile name and job property.

  If outfileOriginal starts with 's' we assume a simulation job, if it starts
  with 'r' we assume a reconstruction job, if starts with "E" we assume
  a stdhepsplit job.

  :param str inputfile: Input file LFN, either \\*.stdhep or \\*.slcio
  :param str outfileOriginal: Output file LFN before change
  :param prodID: Production ID
  :type prodID: `str`, `int`
  :param jobID: jobID
  :type jobID: `str`, `int`
  :returns: Full LFN to changed output file
  """
  finp = FilenameEncoder()
  inpitem = decodeFilename(inputfile)

  origitem = decodeFilename(outfileOriginal)
  originalOutputBaseName = os.path.basename(outfileOriginal)

  outfile = ""
  if originalOutputBaseName.startswith("s"):
    inpitem["s"] = origitem["s"]
    inpitem["m"] = origitem["m"]
    inpitem["d"] = "sim"
    inpitem["t"] = str(prodID).zfill(8)
    inpitem["j"] = str(jobID)
    outfile = finp.convert("sim", "file", inpitem)
  elif originalOutputBaseName.startswith("r"):
    inpitem["r"] = origitem["r"]
    inpitem["m"] = origitem["m"] if "m" in origitem else inpitem["m"]  # Use model name defined by reconstruction
    inpitem["d"] = origitem["d"]
    inpitem["t"] = str(prodID).zfill(8)
    if "n" not in inpitem:  # For DBD sim files ad input
      inpitem["n"] = inpitem["j"] if "j" in inpitem else "0"
    inpitem["j"] = str(jobID)  # Allways use jobID given by production.
    outfile = finp.convert(origitem["d"], "file", inpitem)
  elif originalOutputBaseName.startswith("E"):
    inpitem["d"] = "gen"
    if "n" not in origitem:
      inpitem["n"] = "001"
    else:
      if "_" not in origitem["n"]:
        inpitem["n"] = origitem["n"]
      else:
        inpitem["n"] += "_" + origitem["n"].split("_", 1)[1]
    inpitem["t"] = str(prodID).zfill(8)
    inpitem["j"] = str(jobID)
    outfile = finp.convert("gen", "file", inpitem)
  else:  # Output as it is if not match above
    outfile = originalOutputBaseName

  basepath = os.path.dirname(outfileOriginal)
  return os.path.join(basepath, outfile)

###############################################################################


def getProdFilename(filename, prodID, jobID, workflow_commons=None):
  """Build the output file names based of local job property.

  If workflow_commons is given and contains a ProductionOutputData entry of
  six.string_types that file is returned.

  :param str filename: File name before change
  :param int prodID: Production ID
  :param int jobID: Job ID
  :param dict workflow_commons: workflow_commons dictionary
  :return: the modified file name
  """
  log = gLogger.getSubLogger("getProdFilename")

  if workflow_commons is not None:
    wfProdOut = workflow_commons.get('ProductionOutputData')
    if wfProdOut and isinstance(wfProdOut, six.string_types) and \
      ';' not in wfProdOut:  # a single prod output file
      outfile = wfProdOut
      return os.path.basename(outfile)

  knownFileTypes = ('.slcio', '.stdhep', '.root', '.lhe')
  outfile = ""
  if filename.endswith(knownFileTypes):
    name = filename.rsplit('.', 1)[0]
    extension = filename.rsplit('.', 1)[1]
    outfile = name + '_' + str(prodID) + '_' + str(jobID) + '.' + extension
  elif '.' in filename:
    name = filename.rsplit('.', 1)[0]
    extension = filename.rsplit('.', 1)[1]
    outfile = name + '_' + str(prodID) + '_' + str(jobID) + '.' + extension
  else:
    log.error('Dealing with untreatable outputfile!', filename)
    raise RuntimeError('Cannot treat this production filename!')
  return outfile

###############################################################################


def resolveIFpaths(inputfiles):
  """Try to find out in which sub-directory are each file. In the future, should be useless if PoolXMLCatalog can be used.

  :param list inputfiles: list of inputfiles
  :returns: S_OK(listOfFilePaths), S_ERROR
  """
  log = gLogger.getSubLogger("ResolveInputFiles")
  listoffiles = []
  string = "Will look for:"
  for myfile in inputfiles:
    if not len(myfile):
      continue
    listoffiles.append(os.path.basename(myfile))
    string += "%s\n" % os.path.basename(myfile)
  string = string.rstrip()
  log.info(string)

  listofpaths = []
  listofdirs = []
  for mydir in os.listdir(os.getcwd()):
    if os.path.isdir(mydir):
      listofdirs.append(mydir)
  filesnotfound = []
  for infile in listoffiles:
    filefound = False
    if os.path.exists(os.path.join(os.getcwd(), infile)):
      listofpaths.append(os.path.join(os.getcwd(), infile))
      filefound = True
    else:
      for mydir in listofdirs:
        if os.path.exists(os.path.join(os.getcwd(), mydir, infile)):
          listofpaths.append(os.path.join(os.getcwd(), mydir, infile))
          listofdirs.remove(mydir)
          filefound = True
          break
    if not filefound:
      filesnotfound.append(infile)
  if filesnotfound:
    log.error('The following files were not found locally:', ', '.join(filesnotfound))
    return S_ERROR('resolveIFPath: Input file(s) not found locally')
  log.verbose("Found all input files")
  return S_OK(listofpaths)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""New Job class, for the new interface. This job class should not be used to create jobs. Use :mod:`~ILCDIRAC.Interfaces.API.NewInterface.UserJob` or :mod:`~ILCDIRAC.Interfaces.API.NewInterface.ProductionJob`.

:author: Stephane Poss
:author: Remi Ete
:author: Ching Bon Lam
"""

from __future__ import absolute_import
import inspect

from DIRAC.Interfaces.API.Job import Job as DiracJob
from DIRAC.Core.Utilities.PromptUser import promptUser
from DIRAC.Core.Workflow.Step import StepDefinition

from DIRAC import S_ERROR, S_OK, gLogger

__RCSID__ = "$Id$"

# pylint: disable=protected-access

LOG = gLogger.getSubLogger(__name__)


class Job(DiracJob):
  """ILCDIRAC job class.

  Inherit most functionality from DIRAC Job class
  """

  def __init__(self, script=None):
    super(Job, self).__init__(script)
    #DiracJob.__init__(self, script)
    self.applicationlist = []
    self.inputsandbox = []
    self.outputsandbox = []
    self.check = True
    self.steps = []
    self.nbevts = 0
    self.energy = 0
    self.oktosubmit = False
    self.setPlatform('x86_64-slc5-gcc43-opt')

  def setSystemConfig(self, platform):
    """Deprecation warning for setSystemConfig.

    .. deprecated:: v23r0

    use :func:`setPlatform`
    """
    LOG.error("""WARNING: setSystemConfig has been deprecated! use

                  setPlatform(platform)

                  instead""")
    self.setPlatform(platform)

  def setInputData(self, lfns):
    """Overload method to cancel it."""
    return self._reportError('%s does not implement setInputData' % self.__class__.__name__)

  def setInputSandbox(self, files):
    """Overload method to cancel it."""
    return self._reportError('This job class does not implement setInputSandbox')

  def setOutputData(self, lfns, _OutputPath='', _OutputSE=''):
    """Overload method to cancel it."""
    return self._reportError('This job class does not implement setOutputData')

  def setOutputSandbox(self, files):
    """Overload method to cancel it."""
    return self._reportError('This job class does not implement setOutputSandbox')

  def setIgnoreApplicationErrors(self):
    """Set a flag for all applications that they should not care about errors."""
    self._addParameter(self.workflow, 'IgnoreAppError', 'JDL', True, 'To ignore application errors')
    return S_OK()

  def dontPromptMe(self):
    """Call this function to automatically confirm job submission.

    >>> job = UserJob()
    >>> job.dontPromptMe()
    >>> ...
    >>> job.submit(diracInstance)
    """
    self.check = False
    return S_OK()

  def submit(self, _dirac=None, _mode='wms'):
    """Method to submit the job.

    Not doing anything by default, so that ProductionJobs
    cannot be submitted by mistake
    """
    return self._reportError("Submit is not available for this job class")

  def checkparams(self, dirac=None):
    """Check job consistency instead of submitting.

    :param dirac: DiracILC instance
    :type dirac: :mod:`~ILCDIRAC.Interfaces.API.DiracILC`
    """
    if not dirac:
      return S_ERROR("Missing dirac instance")
    res = self._addToWorkflow()
    if not res['OK']:
      return res
    self.oktosubmit = True
    return dirac.checkparams(self)

  def _askUser(self):
    """Private function.

    Called from :mod:`~ILCDIRAC.Interfaces.API.DiracILC` class to prompt the user
    """
    if not self.check:
      return S_OK()
    for app in self.applicationlist:
      LOG.notice(app)
      app.listAttributes()
      LOG.notice("\n")
    res = promptUser('Proceed and submit job(s)?', logger=LOG)
    if not res['OK']:
      return S_ERROR("User did not validate")
    if res['Value'] == 'n':
      return S_ERROR("User did not validate")
    # no more debug output in further loops
    self.check = False
    return S_OK()

  def setConfigPackage(self, appName, version):
    """Define the config package to obtain at job run time.

    :param str appName: name of the ConfigPackage, e.g. 'ClicConfig'
    :param str version: version of the ConfigPackage
    """
    self._addSoftware(appName.lower(), version)
    self._addParameter(self.workflow, appName + 'Package', 'JDL', appName + version, appName + 'package')
    return S_OK()

  def setCLICConfig(self, version):
    """Define the CLIC Configuration package to obtain.

    Copies steering files from CLIC Configuration folder to working directory

    :param str version: version string, e.g.: 'ILCSoft-2017-07-27'
    """
    return self.setConfigPackage('ClicConfig', version)

  def append(self, application):
    """Helper function.

    This is the main part: call for every application

    :param application: Application instance
    :type application: :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Application`
    """

    res = application._analyseJob(self)
    if not res['OK']:
      return res

    res = application._checkConsistency(self)
    if not res['OK']:
      LOG.error("%s failed to check its consistency:" % application, "%s" % (res['Message']))
      return S_ERROR("%s failed to check its consistency: %s" % (application, res['Message']))

    res = self._jobSpecificParams(application)
    if not res['OK']:
      LOG.error("%s failed job specific checks:" % application, "%s" % (res['Message']))
      return S_ERROR("%s failed job specific checks: %s" % (application, res['Message']))

    res = application._checkFinalConsistency()
    if not res['OK']:
      LOG.error("%s failed to check its consistency:" % application, "%s" % (res['Message']))
      return S_ERROR("%s failed to check its consistency: %s" % (application, res['Message']))

    # Once the consistency has been checked, we can add the application to the list of apps.
    self.applicationlist.append(application)
    # Get the application's sandbox and add it to the job's
    for isb in application.inputSB:
      if isb not in self.inputsandbox:
        self.inputsandbox.append(isb)
    # self.inputsandbox.extend(application.inputSB)

    # Now we can create the step and add it to the workflow
    # First we need a unique name, let's use the application name and step number
    #stepname = "%s_step_%s"%(application.appname,self.stepCount)
    #stepdefinition = StepDefinition(stepname)
    # self.steps.append(stepdefinition)

    # Set the modules needed by the application
#    res = self._jobSpecificModules(application,stepdefinition)
#    if not res['OK']:
#      LOG.error("Failed to add modules: %s"%res['Message'])
#      return S_ERROR("Failed to add modules: %s"%res['Message'])
#
#    ### add the parameters to  the step
#    res = application._addParametersToStep(stepdefinition)
#    if not res['OK']:
#      LOG.error("Failed to add parameters: %s"%res['Message'])
#      return S_ERROR("Failed to add parameters: %s"%res['Message'])
#
#    ##Now the step is defined, let's add it to the workflow
#    self.workflow.addStep(stepdefinition)
#
#    ###Now we need to get a step instance object to set the parameters' values
#    stepInstance = self.workflow.createStepInstance(stepdefinition.getType(),stepname)
#
#    ##Set the parameters values to the step instance
#    res = application._setStepParametersValues(stepInstance)
#    if not res['OK']:
#      LOG.error("Failed to resolve parameters values: %s"%res['Message'])
#      return S_ERROR("Failed to resolve parameters values: %s"%res['Message'])
#
#    res = application._resolveLinkedStepParameters(stepInstance)
#    if not res['OK']:
#      LOG.error("Failed to resolve linked parameters: %s"%res['Message'])
#      return S_ERROR("Failed to resolve linked parameters: %s"%res['Message'])
#    #Now prevent overwriting of parameter values.
#    application._addedtojob()
#
#    self._addParameter(self.workflow, 'TotalSteps', 'String', self.stepCount, 'Total number of steps')
    if application.numberOfEvents:
      self._addParameter(self.workflow, 'NbOfEvts', 'int', application.numberOfEvents, "Number of events to process")

    # Finally, add the software packages if needed
    if application.appname and application.version:
      self._addSoftware(application.appname, application.version)

    # Pass ApplicationErrors to job, will be checked at submission time
    self.errorDict.update(application._errorDict)

    return S_OK()

  def _addToWorkflow(self):
    """This is called just before submission.

    It creates the actual workflow.
    The linking of parameters can only be done here
    """
    for application in self.applicationlist:
      # Start by defining step number
      self.stepCount += 1

      res = application._analyseJob(self)
      if not res['OK']:
        return res

      res = application._checkWorkflowConsistency()
      if not res['OK']:
        LOG.error("%s failed to check its consistency:" % application, "%s" % res['Message'])
        return S_ERROR("%s failed to check its consistency: %s" % (application, res['Message']))

      # Now we can create the step and add it to the workflow
      # First we need a unique name, let's use the application name and step number
      stepname = "%s_step_%s" % (application.appname, self.stepCount)
      stepdefinition = StepDefinition(stepname)
      self.steps.append(stepdefinition)

      # Set the modules needed by the application
      res = self._jobSpecificModules(application, stepdefinition)
      if not res['OK']:
        LOG.error("Failed to add modules:", "%s" % res['Message'])
        return S_ERROR("Failed to add modules: %s" % res['Message'])

      # add the parameters to  the step
      res = application._addParametersToStep(stepdefinition)
      if not res['OK']:
        LOG.error("Failed to add parameters:", "%s" % res['Message'])
        return S_ERROR("Failed to add parameters: %s" % res['Message'])

      # Now the step is defined, let's add it to the workflow
      self.workflow.addStep(stepdefinition)

      # Now we need to get a step instance object to set the parameters' values
      stepInstance = self.workflow.createStepInstance(stepdefinition.getType(), stepname)

      # Set the parameters values to the step instance
      res = application._setStepParametersValues(stepInstance)
      if not res['OK']:
        LOG.error("Failed to resolve parameters values:", "%s" % res['Message'])
        return S_ERROR("Failed to resolve parameters values: %s" % res['Message'])

      res = application._resolveLinkedStepParameters(stepInstance)
      if not res['OK']:
        LOG.error("Failed to resolve linked parameters:", "%s" % res['Message'])
        return S_ERROR("Failed to resolve linked parameters: %s" % res['Message'])
      # Now prevent overwriting of parameter values.
      application._addedtojob()

      self._addParameter(self.workflow, 'TotalSteps', 'String', self.stepCount, 'Total number of steps')

    return S_OK()

  def _jobSpecificModules(self, application, step):  # pylint: disable=no-self-use
    """Returns the list of the job specific modules for the passed application.

    Is overloaded in
    ProductionJob class. UserJob uses the default.
    """
    return application._userjobmodules(step)

  def _jobSpecificParams(self, application):
    """Every type of job has to reimplement this method.

    By default, just set the log file if not
    provided and the energy.
    """
    if not application.logFile:
      logf = application.appname
      if application.version:
        logf += "_" + application.version
      logf += "_Step_%s.log" % (len(self.applicationlist) + 1)
      application.setLogFile(logf)

    if self.energy:
      if not application.energy:
        application.setEnergy(self.energy)
      elif application.energy != self.energy:
        return S_ERROR("You have to use always the same energy per job.")
    else:
      if application.energy:
        self.energy = application.energy
      else:
        LOG.warn("Energy not set for this step")
      #  return S_ERROR("Energy must be set somewhere.")

    if self.energy:
      self._addParameter(self.workflow, "Energy", "float", self.energy, "Energy used")
    return S_OK()

  def _addSoftware(self, appName, appVersion):
    """Private method."""

    currentApp = "%s.%s" % (appName.lower(), appVersion)
    swPackages = 'SoftwarePackages'
    description = 'ILC Software Packages to be installed'

    if not self.workflow.findParameter(swPackages):
      self._addParameter(self.workflow, swPackages, 'JDL', currentApp, description)
    else:
      apps = self.workflow.findParameter(swPackages).getValue()

      if currentApp not in apps.split(';'):
        apps += ';' + currentApp

      self._addParameter(self.workflow, swPackages, 'JDL', apps, description)

  def _checkArgs(self, argNamesAndTypes):
    """Private method to check the validity of the parameters."""

    # inspect.stack()[1][0] returns the frame object ([0]) of the caller
    # function (stack()[1]).
    # The frame object is required for getargvalues. Getargvalues returns
    # a tuple with four items. The fourth item ([3]) contains the local
    # variables in a dict.

    args = inspect.getargvalues(inspect.stack()[1][0])[3]

    #

    for argName, argType in argNamesAndTypes.items():

      if argName not in args:
        self._reportError('Method does not contain argument \'%s\'' % argName,
                           __name__,
                           **self._getArgsDict(1)
                         )
        continue

      if not isinstance(args[argName], argType):
        self._reportError('Argument \'%s\' is not of type %s' % (argName, argType),
                           __name__,
                           **self._getArgsDict(1)
                         )

  def _getArgsDict(self, level=0):  # pylint: disable=no-self-use
    """Private method."""

    # Add one to stack level such that we take the caller function as the
    # reference point for 'level'

    level += 1

    #

    args = inspect.getargvalues(inspect.stack()[level][0])
    adict = {}

    for arg in args[0]:

      if arg == "self":
        continue

      # args[3] contains the 'local' variables

      adict[arg] = args[3][arg]

    return adict



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Helper Functions for Job Interfaces."""


def getValue(value, conversion=None, typeToCheck=None):
  """returns the first entry, if it is a list, or the value otherwise.

  :param value: value to check
  :param conversion: type to convert the value to, callable
  :type conversion: ``callable``
  :param typeToCheck: class the parameter should be an instance of
  :param typeToCheck: ``class``
  """
  newValue = None
  if isinstance(value, list):
    newValue = value[0]
  elif typeToCheck is None or isinstance(value, typeToCheck):
    newValue = value

  if conversion is not None and newValue is not None:
    newValue = conversion(newValue)

  return newValue



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the StdHepCut module."""

from __future__ import absolute_import
import unittest
from mock import patch, mock_open, MagicMock as Mock

from Tests.Utilities.FileUtils import FileUtil
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals, assertMockCalls
from ILCDIRAC.Workflow.Modules.StdHepCut import StdHepCut
from DIRAC import S_OK, S_ERROR

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.StdHepCut'


class StdHepCutTestCase(unittest.TestCase):
  """Contains tests for the StdHepCut class."""

  def setUp(self):
    """set up the objects."""
    self.shc = StdHepCut()

  def test_applicationspecificinputs_nooutputfile(self):
    with patch('%s.os.listdir' % MODULE_NAME, new=Mock(return_value=['something.txt', 'something_else.stdhe', 'stdhep.io'])):
      assertDiracFailsWith(self.shc.applicationSpecificInputs(), 'could not find suitable outputfile name', self)
      assertEqualsImproved(self.shc.OutputFile, '', self)

  def test_applicationspecificinputs(self):
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    self.shc.inlineCuts = 'first_line;Some more content of this file; @end \n of% fi/le'
    with patch('%s.os.listdir' % MODULE_NAME, new=Mock(return_value=['something.txt', 'myoutputfile.stdhep', 'stdhep.io'])), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo:
      mo.side_effect = (h for h in handles)
      assertDiracSucceeds(self.shc.applicationSpecificInputs(), self)
      assertEqualsImproved((self.shc.OutputFile, self.shc.SteeringFile),
                            ('myoutputfile_reduced.stdhep', 'cuts_local.txt'), self)
    mo.assert_called_once_with('cuts_local.txt', 'w')
    assertEqualsImproved(len(handles), 1, self)
    handles[0].write.assert_any_call('first_line\nSome more content of this file\n @end \n of% fi/le')
    assert handles[0].close.called

  def test_applicationspecificinputs_useworkflow(self):
    """Get the OutputFile from the workflow_commons dictionary, don't write to file."""
    self.shc.workflow_commons['IS_PROD'] = True
    self.shc.workflow_commons['ProductionOutputData'] = ';/some/folder/with/afile.txt; GENfile2;/mydir/folder/file3_Gen_experiment1.stdhep;ignoreme   me too; last_file\n'
    self.shc.OutputFile = '1'
    with patch('%s.open' % MODULE_NAME, new=Mock()) as open_mock:
      assertDiracSucceeds(self.shc.applicationSpecificInputs(), self)
      assertEqualsImproved((self.shc.OutputFile, self.shc.SteeringFile),
                            ('file3_Gen_experiment1.stdhep', ''), self)
      self.assertFalse(open_mock.called)

  def test_applicationspecificinputs_sourcesempty(self):
    """Impossible to get the outputfile, use existing one."""
    self.shc.workflow_commons['IS_PROD'] = True
    self.shc.workflow_commons['ProductionOutputData'] = ';/some/folder/with/afile.txt; GENfile2;ignoreme   me too; last_file\n'
    self.shc.OutputFile = 'testFile_dontchangeme'
    with patch('%s.open' % MODULE_NAME, new=Mock()) as open_mock:
      assertDiracSucceeds(self.shc.applicationSpecificInputs(), self)
      assertEqualsImproved((self.shc.OutputFile, self.shc.SteeringFile),
                            ('testFile_dontchangeme', ''), self)
      self.assertFalse(open_mock.called)

  def test_applicationspecificinputs_(self):
    """Guess outputname (using the job properties) using the ILCDIRAC method."""
    self.shc.workflow_commons['IS_PROD'] = True
    self.shc.workflow_commons['PRODUCTION_ID'] = 1498
    self.shc.workflow_commons['JOB_ID'] = 134820
    self.shc.OutputFile = '1'
    with patch('%s.open' % MODULE_NAME, new=Mock()) as open_mock, \
         patch('%s.getProdFilename' % MODULE_NAME, new=Mock(return_value='myoutput_test')) as getname_mock:
      assertDiracSucceeds(self.shc.applicationSpecificInputs(), self)
      assertEqualsImproved((self.shc.OutputFile, self.shc.SteeringFile),
                            ('myoutput_test', ''), self)
      self.assertFalse(open_mock.called)
      getname_mock.assert_called_once_with('1', 1498, 134820)

  def test_preparescript(self):
    exists_dict = {'TestApp_vT_Run_148.sh': True, './lib': True}
    (self.shc.platform,
     self.shc.applicationName,
     self.shc.applicationVersion,
     self.shc.STEP_NUMBER,
     self.shc.MaxNbEvts,
     self.shc.OutputFile,
     self.shc.SteeringFile) = ('testPlatformV1',
                               'TestApp',
                               'vT',
                               148,
                               13,
                               'test_OF.ile',
                               'steer_test.file')
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/my/testsoft/dir1/')) as getldlibs_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo:
      mo.side_effect = (h for h in handles)
      self.shc.prepareScript('test_software/dir')
      remove_mock.assert_called_once_with('TestApp_vT_Run_148.sh')
      getldlibs_mock.assert_called_once_with('testPlatformV1', 'TestApp', 'vT')
      assertMockCalls(exists_mock, ['./lib', 'TestApp_vT_Run_148.sh'], self)
      mo.assert_called_once_with('TestApp_vT_Run_148.sh', 'w')
      assertEqualsImproved(len(handles), 1, self)
      assertMockCalls(handles[0].write, [
          '#!/bin/sh \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'declare -x PATH=test_software/dir:$PATH\n',
          'declare -x LD_LIBRARY_PATH=./lib:test_software/dir/lib:/my/testsoft/dir1/\n',
          'env | sort >> localEnv.log\n', 'echo =============================\n',
          'stdhepCut -m 13 -o test_OF.ile -c steer_test.file *.stdhep\n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      assert handles[0].close.called

  def test_preparescript_othercase(self):
    exists_dict = {'TestApp_vT_Run_148.sh': False, './lib': False}
    (self.shc.platform,
     self.shc.applicationName,
     self.shc.applicationVersion,
     self.shc.STEP_NUMBER,
     self.shc.MaxNbEvts,
     self.shc.OutputFile,
     self.shc.SteeringFile) = ('testPlatformV1',
                               'TestApp',
                               'vT',
                               148,
                               0,
                               'test_OF.ile',
                               'steer_test.file')
    self.shc.fileMask = 'My*.stdhep'
    file_contents = [[]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/my/testsoft/dir1/')) as getldlibs_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo:
      mo.side_effect = (h for h in handles)
      self.shc.prepareScript('test_software/dir')
      self.assertFalse(remove_mock.called)
      getldlibs_mock.assert_called_once_with('testPlatformV1', 'TestApp', 'vT')
      assertMockCalls(exists_mock, ['./lib', 'TestApp_vT_Run_148.sh'], self)
      mo.assert_called_once_with('TestApp_vT_Run_148.sh', 'w')
      assertEqualsImproved(len(handles), 1, self)
      assertMockCalls(handles[0].write, [
          '#!/bin/sh \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'declare -x PATH=test_software/dir:$PATH\n',
          'declare -x LD_LIBRARY_PATH=test_software/dir/lib:/my/testsoft/dir1/\n',
          'env | sort >> localEnv.log\n', 'echo =============================\n',
          "stdhepCut  -o test_OF.ile -c steer_test.file My*.stdhep\n",
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      assert handles[0].close.called

  def test_runit_statusbad(self):
    self.shc.workflowStatus['OK'] = True
    self.shc.workflowStatus['OK'] = False
    self.shc.applicationName = 'testApp'
    assertDiracSucceedsWith_equals(self.shc.runIt(),
                                    'testApp should not proceed as previous step did not end properly', self)

  def test_runit_nooutputfile(self):
    assertDiracFailsWith(self.shc.runIt(), 'outputfile not specified', self)

  def test_runit_getswfolder_fails(self):
    self.shc.OutputFile = 'something'
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_ERROR('getsw_test'))) as getfolder_mock:
      assertDiracFailsWith(self.shc.runIt(), 'getsw_test', self)
      getfolder_mock.assert_called_once_with('testPlatform', 'AppTestName', 'vT')

  def test_runit_getsteeringdir_fails(self):
    self.shc.OutputFile = 'something'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_ERROR('getsteeringdir_err_test'))) as getsteering_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=False)) as exists_mock:
      assertDiracFailsWith(self.shc.runIt(), 'getsteeringdir_err_test', self)
      getsteering_mock.assert_called_once_with('testPlatform', 'AppTestName', 'vT')
      exists_mock.assert_called_once_with('SteerFile.testme')

  def test_runit_copy_steeringfile_fails(self):
    self.shc.OutputFile = 'something'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_OK('/my/steer/dir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=[False, True])) as exists_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('copy_test_err'))) as copy_mock:
      assertDiracFailsWith(self.shc.runIt(), 'failed to access file steerfile.testme: copy_test_err', self)
      assertMockCalls(exists_mock, ['SteerFile.testme', '/my/steer/dir/SteerFile.testme'], self)
      copy_mock.assert_called_once_with('/my/steer/dir/SteerFile.testme', './SteerFile.testme')

  def test_runit_missinglog(self):
    exists_dict = {'SteerFile.testme': True, 'myAppTestLog.log': False}
    open_mock = Mock()
    open_mock.readlines.side_effect = ['line1\n', 'line2\n', 'newline\n', '\n', 'ok\n']
    self.shc.scriptName = 'my_test_script.sh'
    self.shc.OutputFile = 'something'
    self.shc.applicationLog = 'myAppTestLog.log'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as open_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK(('status', )))) as shell_mock, \
         patch('%s.StdHepCut.prepareScript' % MODULE_NAME, new=Mock()) as prep_mock:
      assertDiracFailsWith(self.shc.runIt(), 'apptestname did not produce the expected log', self)
      self.assertFalse(remove_mock.called)
      shell_mock.assert_called_once_with(0, 'sh -c "./my_test_script.sh"',
                                          callbackFunction=self.shc.redirectLogOutput, bufferLimit=20971520)
      chmod_mock.assert_called_once_with('my_test_script.sh', 0o755)
      prep_mock.assert_called_once_with('SoftDir')

  def test_runit_missing_numbers(self):
    exists_dict = {'/my/steer/dir/SteerFile.testme': False, 'SteerFile.testme': False,
                    'myAppTestLog.log': True}
    open_mock = Mock()
    open_mock.readlines.side_effect = ['line1\n', 'line2\n', 'newline\n', '\n', 'ok\n']
    open_mock.__enter__.return_value = ['someline\n', 'otherline']
    self.shc.applicationLog = 'myAppTestLog.log'
    self.shc.OutputFile = 'something'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as open_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()), \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK(('status', )))), \
         patch('%s.StdHepCut.prepareScript' % MODULE_NAME, new=Mock()), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_OK('/my/steer/dir'))):
      assertDiracFailsWith(self.shc.runIt(), 'apptestname exited with status 1', self)
      remove_mock.assert_called_once_with('myAppTestLog.log')

  def test_runit_complete(self):
    exists_dict = {'/my/steer/dir/SteerFile.testme': False, 'SteerFile.testme': False,
                    'myAppTestLog.log': True}
    open_mock = Mock()
    open_mock.readlines.side_effect = ['line1\n', 'line2\n', 'newline\n', '\n', 'ok\n']
    open_mock.__enter__.return_value = ['Events kept 12', 'Events passing cuts 2984', 'Events total 2996']
    self.shc.applicationLog = 'myAppTestLog.log'
    self.shc.OutputFile = 'something'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    self.shc.workflow_commons['Luminosity'] = 13.0
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as open_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()), \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK((0, )))), \
         patch('%s.StdHepCut.prepareScript' % MODULE_NAME, new=Mock()), \
         patch('%s.getSteeringFileDirName' % MODULE_NAME, new=Mock(return_value=S_OK('/my/steer/dir'))):
      assertDiracSucceedsWith_equals(self.shc.runIt(), 'AppTestName vT Successful', self)
      remove_mock.assert_called_once_with('myAppTestLog.log')
      assertEqualsImproved(self.shc.workflow_commons['Luminosity'], 13.0 * (1. * 12 / 2984), self)
      assertEqualsImproved(self.shc.workflow_commons['Info'],
                            {'stdhepcut': {'Reduction': 1. * 12 / 2984,
                                              'CutEfficiency': 1. * 2984 / 2996}}, self)

  def test_runit_othercases(self):
    exists_dict = {'SteerFile.testme': True, 'myAppTestLog.log': False}
    open_mock = Mock()
    open_mock.readlines.side_effect = ['line1\n', 'line2\n', 'newline\n', '\n', 'ok\n']
    open_mock.__enter__.return_value = ['Events kept 23', 'Events passing cuts 14', 'Events total 37']
    self.shc.scriptName = 'my_test_script.sh'
    self.shc.ignoreapperrors = True
    self.shc.OutputFile = 'something'
    self.shc.applicationLog = 'myAppTestLog.log'
    self.shc.SteeringFile = '/my/dir/SteerFile.testme'
    self.shc.workflow_commons['Info'] = {'some_entry': 'rememberMe'}
    self.shc.MaxNbEvts = 100
    (self.shc.platform, self.shc.applicationName, self.shc.applicationVersion) = ('testPlatform',
                                                                                     'AppTestName', 'vT')
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('SoftDir'))), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as open_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()), \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()), \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK((0, )))), \
         patch('%s.StdHepCut.prepareScript' % MODULE_NAME, new=Mock()):
      assertDiracSucceedsWith_equals(self.shc.runIt(), 'AppTestName vT Successful', self)
      assertEqualsImproved(self.shc.workflow_commons['Info'],
                            {'some_entry': 'rememberMe', 'stdhepcut':
                              {'Reduction': 1. * 23 / 14, 'CutEfficiency': 1. * 14 / 37}}, self)
      assert 'Luminosity' not in self.shc.workflow_commons



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create Chain of productions for ILD.

:author: S. Poss, A.Sailer, C. Calancha
:since: Mar 26, 2012
"""

# pylint: disable=invalid-name, wrong-import-position, bad-whitespace, line-too-long
# pylint: disable=bad-indentation
from __future__ import print_function
from __future__ import absolute_import
import os
import pprint

from DIRAC.Core.Base import Script
Script.parseCommandLine()

from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

from ILCDIRAC.Interfaces.API.NewInterface.ILDProductionJobOpt2017 import ILDProductionJobOpt2017
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Mokka, Marlin, OverlayInput, DDSim
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLCIOSplit, StdHepSplit


####################################################################################
'''
Example to run ILDProductionChainOpt2017.py. This script creates 2 transformations
Mokka  or DD4Sim simulation, and Marlin reconsturuction with Overlay.

stdhepsplit won't work for the moment due to an innconsistensy
in creating output filename. This example output sim/rec output to
a temporary directory in /ilc/prod/ild/test/temp1

Simulation and recconstruction step could process more than one GenProcessID.
prodid meta key should be defined to select input file properly.

Define Input serch path, output and input file path properly so that
proper files can be found and processed. This script used prodid = 500002,
for input data query. prodid=500002 is selected, because it will be not used
for the time being.  prodid should be defined to the directory, not as a file meta key
because it is defined as a indexed meta key of directory.

This script limits nb_task for sim and rec to 5. This should be increased to process
more files. dirac-ilc-add-tasks-to-prod command can be used for this purpose.

'''
#####################################################################################


dryrun = False
# TODO: add evttype to the ProdGroup
analysis = 'ILD-Opt2017-Test'  # Some analysis: the prods will belong to the ProdGroup
my_evttype = '3f'
my_evtclass = '3f'
selectedfile = 0
prodid = 500002
genprocessname = 'ea_lvv'
process = ''
energy = 500.  # This is mostly needed to define easily the steering files and the overlay parameters
analysis += '_' + my_evttype

meta_energy = str(int(energy))  # This is needed for the meta data search below

# for the overlay: using DBD numbers
BXOverlay = 1
GGToHadInt250 = 0.2
GGToHadInt350 = 0.33
GGToHadInt500 = 1.7
GGToHadInt1000 = 4.1

MarlinVer = "ILCSoft-01-19_gcc48"
DDSimVer = "ILCSoft-01-19_gcc48"
DDSimILDConfig = "v01-19_lcgeo"

ILDConfig = ''  # Set below for different energies
MokkaVer = "080003"
# MokkaILDConfig = "v01-17-11-p01"
MokkaILDConfig = "v01-14-01-p00"

banned_sites = [""]
# do not register anything nor create anything.
# Should be used once the splitting-at-stdhep-level prods are submitted.

detectorModel = 'ILD_o1_v05'  # OR anything valid, but be careful with the overlay, the files need to exist
dbslice = "mokka-08-00-dbdump.sql"

machineParameters = 'TDR_ws'

UseDD4hepGeometry = False


# This needs to be adapted when using lcgeo geometry
# if energy == 1000.:
#   # ILDConfig         = 'v01-14-01-p00' # this is wrong!
#   ILDConfig         = ILDConfig or 'v01-16-p03'
#   machineParameters = 'B1b_ws'
# elif energy == 500.:
#   ILDConfig     = ILDConfig or 'v01-16-p05_500'
# elif energy == 350.:
#   ILDConfig     = ILDConfig or 'v01-16-p09_350'
# elif energy == 250.:
#   ILDConfig     = ILDConfig or 'v01-16-p10_250'
# else:
#   print "ILDConfig ILD: No ILDConfig defined for this energy (%.1f GeV)"%energy

# ILDConfig for Marlin with or without DD4hep Geometry
ILDConfigRec_DD = "v01-19_lcgeo"
MarlinVer_DD = "ILCSoft-01-19_gcc48"
# ILDConfigRec    = "v01-17-11-p01"
# MarlinVer_DBD = "ILCSoft-01-17-11_gcc48"
ILDConfigRec = "v01-16-p05_500"
MarlinVer_DBD = "v01-16-02"

ILDConfig = ILDConfigRec_DD if UseDD4hepGeometry else ILDConfigRec
ILDConfigSim = DDSimILDConfig if UseDD4hepGeometry else MokkaILDConfig
MarlinVer = MarlinVer_DD if UseDD4hepGeometry else MarlinVer_DBD

additional_name = '_' + analysis + '_' + genprocessname + '_20170411_1_' + str(selectedfile) + '_ildconfig-' + ILDConfig

energyMachinePars = meta_energy + '-' + machineParameters
# Following variables avoid output from stdhepsplit being used
# as input for the same production. Also speed up the job submission.

# basepath = Operations().getValue( '/Production/ILC_ILD/BasePath', '/ilc/prod/ilc/mc-dbd/ild' ).rstrip("/")
# matchToInput_stdhepsplit = '/ilc/prod/ilc/mc-dbd/generated/' + energyMachinePars + '/' + my_evtclass
# matchToInput_mokka       = '/ilc/prod/ilc/mc-dbd.generated/' + energyMachinePars + '/' + my_evttype
# matchToInput_marlin      = basepath + "sim/" + energyMachinePars + '/' + my_evttype + '/' + detectorModel + '/' + ILDConfigSim
# recPath = '/ilc/prod/ilc/mc-dbd/ild/'
# dstPath = '/ilc/prod/ilc/mc-dbd.log/ild/'

# grand_base = '/ilc/prod/ilc/'
grand_base = '/ilc/prod/ilc/ild/test/temp1/'  # Test area for development
basepath = grand_base + 'mc-opt/ild/'
diskpath = grand_base + 'mc-opt.disk/ild/'
my_basepath = basepath  # my_basepath is a base path for Mokka and Marlin

matchToInput_stdhepsplit = '/ilc/prod/ilc/mc-dbd/generated/' + energyMachinePars + '/' + my_evtclass
# matchToInput_stdhepsplit is where input stdhep files are for stdhepsplit
# valid when stdhepsplit production is included.
stdhepsplit_basepath = diskpath + 'splitted/'  # Output directory of stdhepsplit, equal to matchToInput_mokka,


matchToInput_mokka = grand_base + 'gensplit/' + energyMachinePars + \
    '/' + my_evttype + '/run002'  # Where to find input for Mokka
# matchToInput_mokka is input directory for simulation. This example uses Mokka input prepared by other script
matchToInput_marlin = basepath + "sim/" + energyMachinePars + '/' + my_evttype + '/' + detectorModel + '/' + ILDConfigSim
recPath = basepath
dstPath = diskpath

# Define output SE depending on expected data size
SE_stdhepsplit = "PNNL-SRM"
SE_sim = "KEK-SRM"
SE_psplit = "PNNL-SRM"
SE_rec = "DESY-SRM"


###LCG_SITE  = "LCG.KEK.jp"
input_sand_box = [""]
# This is where magic happens
meta = {}

meta['Datatype'] = 'gensplit'  # MOKKA or stdhepsplit or MOKKA+MARLIN
# meta['Datatype']      = 'SIM' # JUST MARLIN / MARLIN_OVERLAY

meta['Energy'] = meta_energy
meta['Machine'] = 'ilc'
meta['GenProcessName'] = genprocessname
meta['MachineParams'] = machineParameters
meta['ProdID'] = prodid

# GenProcessID or ProcessID
if meta.get('Datatype', None) in ('gen', 'gensplit'):
  #meta['GenProcessID'] = process
  pass
else:
  meta['ProcessID'] = process
  # These parameters automatically retrieved if you run Mokka. If
  # running standalone Marlin you need to specify them
  meta['DetectorModel'] = detectorModel
  meta['EvtClass'] = my_evttype
  meta['MachineParams'] = machineParameters
  meta['ProdID'] = prodid

# inputFileFolder= "/ilc/prod/ilc/ild/test/temp1/gensplit/500-TDR_ws/3f/run001"

# DoSplit at stdhep level
activesplitstdhep = False
nbevtsperfilestdhep = 500
nbtasks_split = -1  # To run over all input stdhep
if activesplitstdhep:
  if selectedfile > 0:
    meta['SelectedFile'] = selectedfile
  else:
    print('ERROR: stdhepsplit requires SelectedFile in the metadata to prevent output being used as input.')
    exit(1)
else:
  if prodid:
    meta['ProdID'] = prodid
  if selectedfile > 0:
    meta['SelectedFile'] = selectedfile
    print('Warning: SelectedFile meta field active: this should only happen when debugging.')

# Do Sim
ild_sim = True
nbtasks_sim = 3
# nbtasks_sim = 10 # create 10 jobs from input meta data query result. -1 to create for all files.
# It can be extended with dirac-ilc-add-tasks-to-prod
# It's possible to get this number automatically by getting the number of events per file (if known)
# nbtasks = math.ceil(number_of_events_to_process/nb_events_per_signal_file) #needs import math

# DoSplit
activesplit = False
nbevtsperfile = 200

# Do Reco with Overlay
ild_rec_ov = True
nbtasks_rec_ov = 3  # See comment on nbtasks_sim
# Do Reco
ild_rec = False  # please, use WITH OVERLAY
nbtasks_rec = -1  # See comment on nbtasks_sim

print("####################################################################################")
if activesplitstdhep:
  print("matchToInput_stdhepsplit (input)      = %s  (Valid when stdhep production is included)"
        % matchToInput_stdhepsplit)
  print("stdhepsplit_basepath (output)         = %s  (Valid when stdhep production is included)"
        % stdhepsplit_basepath)

print("matchToInput_mokka (input)            = " + matchToInput_mokka)
print("matchToInput_marlin (input)           = " + matchToInput_marlin)
print("Basepath for sim&rec (output)         = " + basepath)
print("Diskpath for stdhepsplit&dst (output) = " + diskpath)
print("Outtput SE : stdhepsplit(%s), SUM(%s), psplit(%s), REC&DST(%s)" % (SE_stdhepsplit, SE_sim, SE_psplit, SE_rec))
print("####################################################################################")
##############################################################################################
# [PART2] Below is not to be touched
##############################################################################################
# Whatever is below is not to be touched... Or at least only when something changes

# Split
stdhepsplit = StdHepSplit()
stdhepsplit.setVersion("V2")
stdhepsplit.setNumberOfEventsPerFile(nbevtsperfilestdhep)

# Simulation ILD
mo = Mokka()
mo.setVersion(MokkaVer)  # SET HERE YOUR MOKKA VERSION, the software will come from the ILDConfig
mo.setDetectorModel(detectorModel)
mo.setSteeringFile("bbudsc_3evt.steer")
# Do not include '.tgz'
mo.setDbSlice(dbslice)

# Simulation ILD
ddsim = None
if UseDD4hepGeometry:
  ddsim = DDSim()
  ddsim.setVersion(DDSimVer)  # SET HERE YOUR MOKKA VERSION, the software will come from the ILDConfig
  ddsim.setDetectorModel(detectorModel)
  ddsim.setSteeringFile("ddsim_steer.py")


# Split
split = SLCIOSplit()
split.setNumberOfEventsPerFile(nbevtsperfile)

# Define the overlay
overlay = OverlayInput()
overlay.setMachine("ilc_dbd")  # Don't touch, this is how the system knows what files to get
overlay.setEnergy(energy)  # Don't touch, this is how the system knows what files to get
overlay.setDetectorModel(detectorModel)  # Don't touch, this is how the system knows what files to get
if energy == 500.:  # here you chose the overlay parameters as this determines how many files you need
  # it does NOT affect the content of the marlin steering file whatsoever, you need to make sure the values
  # there are correct. Only the file names are handled properly so that you don't need to care
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt500)
  overlay.setBkgEvtType("aa_lowpt2")  # lowpt2: correct number of events (500),
  # not increased to 2500 to reduce number
  # of downloaded files
elif energy == 1000.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt1000)
  overlay.setBkgEvtType("aa_lowpt")
elif energy == 350.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt350)
  overlay.setBkgEvtType("aa_lowpt")
elif energy == 250.:
  overlay.setBXOverlay(BXOverlay)
  overlay.setGGToHadInt(GGToHadInt250)
  overlay.setBkgEvtType("aa_lowpt")

else:
  print("Overlay ILD: No overlay parameters defined for this energy")

# Reconstruction ILD with overlay
mao = Marlin()
mao.setDebug()
mao.setVersion(MarlinVer)  # PUT HERE YOUR MARLIN VERSION
if ild_rec_ov:
  if energy in [250.0, 350.0, 500.0, 1000.0]:
    if UseDD4hepGeometry:
      mao.setSteeringFile("bbudsc_3evt_stdreco_dd4hep.xml")
      mao.setGearFile("GearOutput.xml")
      mao.setDetectorModel(detectorModel)
    else:
      mao.setSteeringFile("bbudsc_3evt_stdreco.xml")
      mao.setGearFile("GearOutput.xml")
  else:
    print("Marlin: No reconstruction suitable for this energy")


# Reconstruction ILD w/o overlay
ma = Marlin()
ma.setDebug()
ma.setVersion(MarlinVer)
ma.setEnergy(energy)
if ild_rec:
  if energy in [250.0, 350.0, 500.0, 1000.0]:
    if UseDD4hepGeometry:
      ma.setSteeringFile("bbudsc_3evt_stdreco_dd4hep.xml")
      ma.setGearFile("GearOutput.xml")
      ma.setDetectorModel(detectorModel)
    else:
      ma.setSteeringFile("bbudsc_3evt_stdreco.xml")
      ma.setGearFile("GearOutput.xml")
  else:
    print("Marlin: No reconstruction suitable for this energy %g" % (energy))


###################################################################################
# HERE WE DEFINE THE PRODUCTIONS
if activesplitstdhep and meta:
  print("################## Createing a production for stdhepsplit. Input meta is")
  pprint.pprint(meta)
  pstdhepsplit = ILDProductionJobOpt2017()
  pstdhepsplit.basepath = stdhepsplit_basepath  # Sailer suggestion
  pstdhepsplit.matchToInput = matchToInput_stdhepsplit
  pstdhepsplit.setDryRun(dryrun)
  # pstdhepsplit.setILDConfig(ILDConfig) ## stdhepsplit does not need ILDConfig
  pstdhepsplit.setEvtClass(my_evtclass)
  pstdhepsplit.setEvtType(my_evttype)
  # pstdhepsplit.setUseSoftTagInPath(False)
  pstdhepsplit.setLogLevel("verbose")
  pstdhepsplit.setProdType('Split_ILD')
  pstdhepsplit.setBannedSites(banned_sites)
  pstdhepsplit.setProdPlugin('Limited')  # exit with error: it seems i need
  # to set the Prod. plugin

  # generated files has not SoftwareTag: we exclude it from inputdataquery, but
  # reinserted at the end of the module to be used in the next simulation module
  tmp_softwaretag_val = ''
  if 'SoftwareTag' in meta:
    tmp_softwaretag_val = meta.pop('SoftwareTag')
    print("'SoftwareTag' found in metadata for pstdhepsplit module: excluded from input data query")

  res = pstdhepsplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.setOutputSE(SE_stdhepsplit)
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  pstdhepsplit.setWorkflowName(wname)
  pstdhepsplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pstdhepsplit.append(stdhepsplit)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.addFinalization(True, True, True, True)
  descrp = "Splitting stdhep files"
  if additional_name:
    descrp += ", %s" % additional_name
  pstdhepsplit.setDescription(descrp)

  res = pstdhepsplit.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pstdhepsplit.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pstdhepsplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.setNbOfTasks(nbtasks_split)
  # As before: get the metadata for this production to input into the next
  meta = pstdhepsplit.getMetadata()
  if tmp_softwaretag_val:  # reinsert SoftwareTag: used in path construction
    meta.update({'SoftwareTag': tmp_softwaretag_val})

  print("### Following meta data were created for subsequent simulation production")
  pprint.pprint(meta)
  print("### Done With Stdhepsplit", "\n" * 5)

if ild_sim and meta:
  print("################## Createing a production for simulation. Input meta is")
  pprint.pprint(meta)
  ####################
  # Define the second production (simulation). Notice the setInputDataQuery call
  pSim = ILDProductionJobOpt2017()
  pSim.basepath = my_basepath
  pSim.matchToInput = matchToInput_mokka
  pSim.setDryRun(dryrun)
  pSim.setProdPlugin('Standard')
  pSim.setILDConfig(ILDConfigSim)
  pSim.setEvtClass(my_evtclass)
  pSim.setUseSoftTagInPath(True)
  pSim.setEvtType(my_evttype)
  pSim.setLogLevel("verbose")
  pSim.setProdType('MCSimulation_ILD')
  pSim.setBannedSites(banned_sites)
  pSim.setInputSandbox(input_sand_box)
  # pSim.setDestination(LCG_SITE)
  pSim.setProdPlugin('Limited')

  res = pSim.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pSim.setOutputSE(SE_sim)
  wname = process + "_" + str(energy) + "_ild_sim"
  wname += additional_name
  pSim.setWorkflowName(wname)
  pSim.setProdGroup(analysis + "_" + str(energy))
  # Add the application
  if UseDD4hepGeometry:
    res = pSim.append(ddsim)
  else:
    res = pSim.append(mo)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pSim.addFinalization(True, True, True, True)
  descrp = "%s model" % detectorModel

  if additional_name:
    descrp += ", %s" % additional_name
  pSim.setDescription(descrp)
  res = pSim.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pSim.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pSim.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pSim.setNbOfTasks(nbtasks_sim)
  # As before: get the metadata for this production to input into the next
  meta = pSim.getMetadata()

  print("### Done With a creation of ild_sim transformation", "\n" * 5)

# Split at slcio level (after sim)
if activesplit and meta:
  print("################## Creating a prduction for sim-slcio split: input meta is")
  pprint.pprint(meta)
  #######################
  # Split the input files.
  psplit = ILDProductionJobOpt2017()
  psplit.setDryRun(dryrun)
  psplit.setCPUTime(30000)
  psplit.setLogLevel("verbose")
  psplit.setProdType('Split')
  psplit.setInputSandbox(input_sand_box)
  res = psplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.setOutputSE(SE_psplit)
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  psplit.setWorkflowName(wname)
  psplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = psplit.append(split)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.addFinalization(True, True, True, True)
  descrp = "Splitting slcio files"
  if additional_name:
    descrp += ", %s" % additional_name
  psplit.setDescription(descrp)

  res = psplit.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = psplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # As before: get the metadata for this production to input into the next
  meta = psplit.getMetadata()

if ild_rec and meta:
  print("################## Creating a production for reconstruction WITHOUT overlay. Input meta is")
  pprint.pprint(meta)
  #######################
  # Define the reconstruction prod
  pma = ILDProductionJobOpt2017()
  pma.setDryRun(dryrun)
  pma.matchToInput = matchToInput_marlin
  pma.setILDConfig(ILDConfigRec)
  pma.setLogLevel("verbose")
  pma.setProdType('MCReconstruction_ILD')
  pma.setEvtType(my_evttype)
  pma.setReconstructionBasePaths(recPath, dstPath)
  pma.setProdPlugin('Limited')
#   pma.setInputSandbox(input_sand_box_marlin)
  pma.setOutputSandbox(["*.log", "*dd4hep.xml", "*.sh", "TaggingEfficiency.root", "PfoAnalysis.root"])

  res = pma.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)

  pma.setOutputSE(SE_rec)
  wname = process + "_" + str(energy) + "_ild_rec"
  wname += additional_name
  pma.setWorkflowName(wname)
  pma.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pma.append(ma)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pma.addFinalization(True, True, True, True)
  descrp = "%s, No overlay" % detectorModel
  if additional_name:
    descrp += ", %s" % additional_name
  pma.setDescription(descrp)

  res = pma.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = pma.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pma.setNbOfTasks(nbtasks_rec)

if ild_rec_ov and meta:
  print("################## Creating a production for reconstruction WITH overlay. Input meta is")
  pprint.pprint(meta)
  #######################
  # Define the reconstruction prod
  pmao = ILDProductionJobOpt2017()
  pmao.matchToInput = matchToInput_marlin
  pmao.setDryRun(dryrun)
  pmao.setILDConfig(ILDConfigRec)
  pmao.setEvtClass(my_evtclass)
  # pmao.setUseSoftTagInPath(False)
  pmao.setEvtType(my_evttype)
  pmao.setLogLevel("verbose")
  pmao.setProdType('MCReconstruction_Overlay_ILD')
  pmao.setBannedSites(banned_sites)
  pmao.setOutputSandbox(["*.log", "*.sh", "*.xml"])
  pmao.setReconstructionBasePaths(recPath, dstPath)
  pmao.setProdPlugin('Limited')

  res = pmao.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.setOutputSE(SE_rec)
  wname = process + "_" + str(energy) + "_ild_rec_overlay"
  wname += additional_name
  pmao.setWorkflowName(wname)
  pmao.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pmao.append(overlay)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # Add the application
  res = pmao.append(mao)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.addFinalization(True, True, True, True)
  descrp = "%s, Overlay" % detectorModel

  if additional_name:
    descrp += ", %s" % additional_name
  pmao.setDescription(descrp)
  res = pmao.createProduction()
  if not res['OK']:
    print(res['Message'])

  res = pmao.setProcessIDInFinalPath()
  if not res['OK']:
    print(res['Message'])

  res = pmao.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pmao.setNbOfTasks(nbtasks_rec_ov)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Register the meta data of the production files.

:since: Sep 8, 2010

:author: sposs
"""

from __future__ import absolute_import
import os

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.Core.Utilities import DEncode
from DIRAC import S_OK, gLogger

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class RegisterOutputData(ModuleBase):
  """At the end of a production Job, we need to register meta data info for the files."""

  def __init__(self):
    super(RegisterOutputData, self).__init__()
    self.version = "RegisterOutputData v1"
    self.commandTimeOut = 10 * 60
    self.enable = True
    self.prodOutputLFNs = []
    self.nbofevents = 0
    self.luminosity = 0
    self.sel_eff = 0
    self.cut_eff = 0
    self.add_info = ''
    self.filecatalog = FileCatalogClient()
    self.filemeta = {}

  def applicationSpecificInputs(self):
    if 'Enable' in self.step_commons:
      self.enable = self.step_commons['Enable']
      if not isinstance(self.enable, bool):
        LOG.warn('Enable flag set to non-boolean value %s, setting to False' % self.enable)
        self.enable = False

    if 'ProductionOutputData' in self.workflow_commons:
      self.prodOutputLFNs = self.workflow_commons['ProductionOutputData'].split(";")
    else:
      self.prodOutputLFNs = []

    self.nbofevents = self.NumberOfEvents
    if 'Luminosity' in self.workflow_commons:
      self.luminosity = self.workflow_commons['Luminosity']

    # Additional info: cross section only for the time being, comes from WHIZARD
    if 'Info' in self.workflow_commons:
      if 'stdhepcut' in self.workflow_commons['Info']:
        self.sel_eff = self.workflow_commons['Info']['stdhepcut']['Reduction']
        self.cut_eff = self.workflow_commons['Info']['stdhepcut']['CutEfficiency']
        del self.workflow_commons['Info']['stdhepcut']
      self.add_info = DEncode.encode(self.workflow_commons['Info']).decode()

    return S_OK('Parameters resolved')

  def execute(self):
    """Run the module."""
    LOG.info('Initializing %s' % self.version)
    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error("failed to resolve input parameters:", result['Message'])
      return result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('No registration of output data metadata attempted')

    if len(self.prodOutputLFNs) == 0:
      LOG.info('No production data found, so no metadata registration to be done')
      return S_OK("No files' metadata to be registered")

    LOG.verbose("Will try to set the metadata for the following files: \n %s" % "\n".join(self.prodOutputLFNs))

    for files in self.prodOutputLFNs:
      metafiles = {}

      if self.nbofevents:
        nbevts = {}
        nbevts['NumberOfEvents'] = self.nbofevents
        if 'file_number_of_event_relation' in self.workflow_commons:
          if os.path.basename(files) in self.workflow_commons['file_number_of_event_relation']:
            nbevts['NumberOfEvents'] = self.workflow_commons['file_number_of_event_relation'][os.path.basename(files)]
        metafiles.update(nbevts)
      if self.luminosity:
        metafiles.update({'Luminosity': self.luminosity})

      if self.sel_eff:
        metafiles.update({'Reduction': self.sel_eff})
      if self.cut_eff:
        metafiles.update({'CutEfficiency': self.cut_eff})
      if self.add_info:
        metafiles.update({'AdditionalInfo': self.add_info})

      elif 'AdditionalInfo' in self.inputdataMeta:
        metafiles.update({'AdditionalInfo': self.inputdataMeta['AdditionalInfo']})

      if 'CrossSection' in self.inputdataMeta:
        metafiles.update({'CrossSection': self.inputdataMeta['CrossSection']})

      if self.WorkflowStartFrom:
        metafiles.update({"FirstEventFromInput": self.WorkflowStartFrom})

      if self.enable:
        res = self.filecatalog.setMetadata(files, metafiles)
        if not res['OK']:
          LOG.error(res['Message'])
          LOG.error('Could not register metadata for %s' % files)
          return res

      LOG.info("Registered %s with tags %s" % (files, metafiles))

      # Now, set the ancestors
      if self.InputData:
        if self.enable:
          res = self.filecatalog.addFileAncestors({files: {'Ancestors': self.InputData}})
          if not res['OK']:
            LOG.error('Registration of Ancestors for %s failed' % files)
            LOG.error('Because of ', res['Message'])
            return res

    return S_OK('Output data metadata registered in catalog')



"""Retrieve Gear information for HCal Ring Digitisation."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
Gear_File_And_Path = sys.argv[2]
Energy_To_Calibrate = sys.argv[3]

Check_Ring_One = 'Off'
Check_Ring_Two = 'Off'

Absorber_Thickness_Ring = 0
Scintillator_Thickness_Ring = 0

Check_EndCap_One = 'Off'
Check_EndCap_Two = 'Off'

Absorber_Thickness_EndCap = 0
Scintillator_Thickness_EndCap = 0

Check_EndCap = 'Off'

with open(Gear_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    for line in searchlines:
        if 'detector name="HcalRing"' in line:
            Check_Ring_One = 'On'
            Check_Ring_Two = 'On'

        if Check_Ring_One == 'On':
            if 'absorberThickness' in line:
                Number = float(find_between(line, 'absorberThickness="', 'e'))
                trimmedLine = find_between(line, 'absorberThickness="', 'Size')
                Power_Of_Ten = float(find_between(trimmedLine, 'e+', '" cell'))
                Absorber_Thickness_Ring = Number * pow(10, Power_Of_Ten)
                Check_Ring_One = 'Off'

        if Check_Ring_Two == 'On':
            if 'Hcal_scintillator_thickness' in line:
                Number = float(find_between(line, 'type="double" value="', 'e'))
                Power_Of_Ten = float(find_between(line, 'e+', '"'))
                Scintillator_Thickness_Ring = Number * pow(10, Power_Of_Ten)
                Check_Ring_Two = 'Off'

    for line in searchlines:
        if 'detector name="HcalEndcap"' in line:
            Check_EndCap_One = 'On'
            Check_EndCap_Two = 'On'

        if Check_EndCap_One == 'On':
            if 'absorberThickness' in line:
                Number = float(find_between(line, 'absorberThickness="', 'e'))
                trimmedLine = find_between(line, 'absorberThickness="', 'Size')
                Power_Of_Ten = float(find_between(trimmedLine, 'e+', '" cell'))
                Absorber_Thickness_EndCap = Number * pow(10, Power_Of_Ten)
                Check_EndCap_One = 'Off'

        if Check_EndCap_Two == 'On':
            if 'Hcal_scintillator_thickness' in line:
                Number = float(find_between(line, 'type="double" value="', 'e'))
                Power_Of_Ten = float(find_between(line, 'e+', '"'))
                Scintillator_Thickness_EndCap = Number * pow(10, Power_Of_Ten)
                Check_EndCap_Two = 'Off'

Calibration_Text = '_____________________________________________________________________________________' + '\n'

Calibration_Text += 'HCal_Ring_Gear_information.py retrieving GEAR information for HCal Ring Digitisation' + '\n'
Calibration_Text += 'For Muons with energy                              :' + str(Energy_To_Calibrate) + ' /GeV \n'
Calibration_Text += 'Absorber_Thickness_Ring                            : ' + str(Absorber_Thickness_Ring) + ' /mm \n'
Calibration_Text += 'Scintillator_Thickness_Ring                        : ' + \
    str(Scintillator_Thickness_Ring) + ' /mm \n'
Calibration_Text += 'Absorber_Thickness_EndCap                          : ' + str(Absorber_Thickness_EndCap) + ' /mm \n'
Calibration_Text += 'Scintillator_Thickness_EndCap                      : ' + \
    str(Scintillator_Thickness_EndCap) + ' /mm \n\n'

print(Calibration_Text)
ratio = (Absorber_Thickness_EndCap * Scintillator_Thickness_Ring) / \
    (Absorber_Thickness_Ring * Scintillator_Thickness_EndCap)

Calibration_Text += 'Ratio used for HCal Ring Digitisation is           : ' + '\n'
Calibration_Text += 'Absorber_Thickness_EndCap x Scintillator_Thickness_Ring' + '\n'
Calibration_Text += '-------------------------------------------------------     = ' + str(ratio) + '\n'
Calibration_Text += 'Scintillator_Thickness_EndCap x Absorber_Thickness_Ring' + '\n\n'

with open(Calibration_File_And_Path, 'a') as myfile:
    myfile.write(Calibration_Text)

print(ratio)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test for WMS clients."""
# pylint: disable=protected-access, missing-docstring, invalid-name, line-too-long

from __future__ import print_function
from __future__ import absolute_import
import pytest

from mock import MagicMock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.WorkloadManagementSystem.Client.DownloadInputData import DownloadInputData

MODULE_NAME = 'ILCDIRAC.WorkloadManagementSystem.Client.DownloadInputData'

# pylint: disable=redefined-outer-name, unused-argument


@pytest.fixture(autouse=True)
def setUpLogger():
  from DIRAC import gLogger
  gLogger.setLevel('DEBUG')


@pytest.fixture
def mockSE(mocker):
  mockObjectSE = MagicMock()
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful': {'/a/lfn/1.txt': {'Cached': 1, 'Accessible': 1}},
                                                    'Failed': {}})
  mockObjectSE.getFile.return_value = S_OK({'Successful': {'/a/lfn/1.txt': {}}, 'Failed': {}})
  mockObjectSE.getStatus.return_value = S_OK({'Read': True, 'DiskSE': True})
  mockObjectSE.status.return_value = {'Read': True, 'DiskSE': True}

  theMockSE = MagicMock()
  theMockSE.return_value = mockObjectSE
  mocker.patch(MODULE_NAME + '.StorageElement', new=theMockSE)
  return theMockSE


@pytest.fixture
def dli():
  theDLI = DownloadInputData({'InputData': [],
                              'Configuration': {'LocalSEList': ['SE_Local']},
                              'InputDataDirectory': 'CWD',
                              'FileCatalog':
                                S_OK({'Successful': {'/a/lfn/1.txt': {'Size': 10,
                                                                      'GUID': 'aGUID',
                                                                      'SE_Local': '',
                                                                      'SE_Remote': '',
                                                                      'SE_Bad': '',
                                                                      'SE_Tape': '',
                                                                      }}})})
  theDLI._DownloadInputData__checkDiskSpace = MagicMock(return_value=S_OK(1000))
  theDLI.availableSEs = ['SE_Local', 'SE_Remote']
  return theDLI


@pytest.fixture
def osPathExists(mocker):
  osPathMock = MagicMock(return_value=True)
  mocker.patch('%s.os.path.exists' % MODULE_NAME, new=osPathMock)
  return osPathMock


@pytest.fixture
def getDiskSpace(mocker):
  gdsMock = MagicMock(return_value=True)
  mocker.patch('%s.getDiskSpace' % MODULE_NAME, new=gdsMock)
  return gdsMock


def test_DLIDownloadFromSE_fail(dli, mockSE, osPathExists):
  osPathExists.return_value = False
  res = dli._downloadFromSE('/a/lfn/1.txt', 'mySE', {'mySE': []}, 'aGuid')
  assert not res['OK']


def test_DLIDownloadFromSE_local(dli, mockSE, osPathExists):
  res = dli._downloadFromSE('/a/lfn/1.txt', 'mySE', {'mySE': []}, 'aGuid')
  assert res['OK'], res
  assert res['Value']['protocol'] == 'LocalData'


def test_DLIDownloadFromSE_realDown(dli, mockSE, osPathExists):
  osPathExists.side_effect = (False, False, True)
  res = dli._downloadFromSE('/a/lfn/1.txt', 'mySE', {'mySE': []}, 'aGuid')
  assert res['OK'], res.get('Message', 'No Error')
  assert res['Value']['protocol'] == 'Downloaded'
  assert 'path' in res['Value']
  assert res['Value']['path'].endswith('1.txt')


def test_DLIDownloadFromSE_realDown_Fail(dli, mockSE, osPathExists):
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFile.return_value = S_OK({'Successful': {},
                                            'Failed': {'/a/lfn/1.txt': {}}})
  mockObjectSE.getStatus.return_value = S_OK({'Read': True, 'DiskSE': True})

  osPathExists.return_value = False
  res = dli._downloadFromSE('/a/lfn/1.txt', 'mySE', {'mySE': []}, 'aGuid')
  assert not res['OK'], res.get('Message', 'No Error')


def test_DLIDownloadFromBestSE_isLocal(dli, mockSE, osPathExists):
  res = dli._downloadFromBestSE('/a/lfn/1.txt', {'mySE': []}, 'aGuid')
  assert res['OK'], res.get('Message', 'No error')
  assert res['Value']['protocol'] == 'LocalData'


def test_DLIDownloadFromBestSE_Fail(dli, mockSE, osPathExists):
  osPathExists.return_value = False
  res = dli._downloadFromBestSE('/a/lfn/1.txt', {'mySE': []}, 'aGuid')
  assert not res['OK']


def test_DLI_execute(dli, mockSE):
  dli._downloadFromSE = MagicMock(return_value=S_OK({'path': '/local/path/1.txt'}))
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res


def test_DLI_execute_getFileMetadata_Fails(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful': {},
                                                    'Failed': {'/a/lfn/1.txt': 'Error Getting MetaData'}})

  dli._downloadFromSE = MagicMock(return_value=S_OK({'path': '/local/path/1.txt'}))
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res
  assert res['Value']['Successful']['/a/lfn/1.txt']['path'] == '/local/path/1.txt', res


def test_DLI_execute_getFileMetadata_Lost(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful':
                                                    {'/a/lfn/1.txt':
                                                     {'Cached': 1, 'Accessible': 0,
                                                      'Lost': True,
                                                      }},
                                                    'Failed': {}})
  dli._downloadFromSE = MagicMock(return_value=S_OK({'path': '/local/path/1.txt'}))
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res
  assert res['Value']['Successful']['/a/lfn/1.txt']['path'] == '/local/path/1.txt', res


def test_DLI_execute_getFileMetadata_Unavailable(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful':
                                                    {'/a/lfn/1.txt':
                                                     {'Cached': 0, 'Accessible': 0,
                                                      'Unavailable': True,
                                                      }},
                                                    'Failed': {}})
  dli._downloadFromSE = MagicMock(return_value=S_OK({'path': '/local/path/1.txt'}))
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res
  assert res['Value']['Successful']['/a/lfn/1.txt']['path'] == '/local/path/1.txt', res


def test_DLI_execute_getFileMetadata_Cached(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful':
                                                    {'/a/lfn/1.txt':
                                                     {'Cached': 0, 'Accessible': 0,
                                                      'Unavailable': False,
                                                      }},
                                                    'Failed': {}})
  dli._downloadFromSE = MagicMock(return_value=S_OK({'path': '/local/path/1.txt'}))
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res
  assert res['Value']['Successful']['/a/lfn/1.txt']['path'] == '/local/path/1.txt', res


def test_DLI_execute_FirstDownFailed(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful':
                                                    {'/a/lfn/1.txt': {'Cached': 1, 'Accessible': 0}},
                                                    'Failed': {}})
  dli._downloadFromSE = MagicMock(side_effect=[S_ERROR('Failed to down'),
                                               S_OK({'path': '/local/path/1.txt'})])
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert not res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Successful'], res
  assert res['Value']['Successful']['/a/lfn/1.txt']['path'] == '/local/path/1.txt', res


def test_DLI_execute_AllDownFailed(dli, mockSE):
  """When getFileMetadata fails for the first SE, we should fall back to the second."""
  mockObjectSE = mockSE.return_value
  mockObjectSE.getFileMetadata.return_value = S_OK({'Successful':
                                                    {'/a/lfn/1.txt': {'Cached': 1, 'Accessible': 0}},
                                                    'Failed': {}})
  dli._downloadFromSE = MagicMock(return_value=S_ERROR('Failed to down'))
  dli._isCache = MagicMock(return_value=True)
  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Failed'], res
  assert res['Value']['Failed'][0] == '/a/lfn/1.txt', res


def test_DLI_execute_NoLocal(mockSE):
  """Data only at the remote SE."""
  dli = DownloadInputData({'InputData': [],
                           'Configuration': {'LocalSEList': ['SE_Local', 'SE_Tape']},
                           'InputDataDirectory': 'CWD',
                           'FileCatalog':
                           S_OK({'Successful': {'/a/lfn/1.txt': {'Size': 10, 'GUID': 'aGUID',
                                                                 'SE_Tape': '',
                                                                 }}})})
  dli._DownloadInputData__checkDiskSpace = MagicMock(return_value=S_OK(1000))
  dli.availableSEs = ['SE_Local', 'SE_Remote', 'SE_Tape']
  mockObjectSE = mockSE.return_value
  statDict = {'Read': True, 'TapeSE': True, 'DiskSE': False}
  mockObjectSE.getStatus.return_value = S_OK(statDict)
  mockObjectSE.status.return_value = statDict
  dli._downloadFromSE = MagicMock(return_value=S_ERROR('Failed to down'))
  dli._isCache = MagicMock(return_value=True)

  res = dli.execute(dataToResolve=['/a/lfn/1.txt'])
  assert res['OK']
  assert res['Value']['Failed']
  assert '/a/lfn/1.txt' in res['Value']['Failed'], res
  assert res['Value']['Failed'][0] == '/a/lfn/1.txt', res


def test_DLI_checkDiskSpace_success(getDiskSpace):
  """CheckDiskSpace function."""
  getDiskSpace.return_value = 10 * 1024  # in MB
  dli = DownloadInputData({'InputData': [],
                           'Configuration': {'LocalSEList': ['SE_Local', 'SE_Tape']},
                           'InputDataDirectory': 'CWD',
                           'FileCatalog':
                           S_OK({'Successful': {'/a/lfn/1.txt': {'Size': 10, 'GUID': 'aGUID',
                                                                 'SE_Tape': '',
                                                                 }}})})
  res = dli._DownloadInputData__checkDiskSpace(totalSize=10)
  assert res['OK']
  assert 'Enough disk space' in res['Value']


def test_DLI_checkDiskSpace_fail(getDiskSpace):
  """CheckDiskSpace function."""
  getDiskSpace.return_value = 10  # in MB
  dli = DownloadInputData({'InputData': [],
                           'Configuration': {'LocalSEList': ['SE_Local', 'SE_Tape']},
                           'InputDataDirectory': 'CWD',
                           'FileCatalog':
                           S_OK({'Successful': {'/a/lfn/1.txt': {'Size': 10, 'GUID': 'aGUID',
                                                                 'SE_Tape': '',
                                                                 }}})})
  res = dli._DownloadInputData__checkDiskSpace(totalSize=10 * 1024 * 1024)
  assert not res['OK']
  assert 'Not enough' in res['Message']



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the StdHepSplit module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceedsWith_equals, assertDiracSucceeds, assertListContentEquals
from ILCDIRAC.Workflow.Modules.StdHepSplit import StdHepSplit
from DIRAC import S_OK, S_ERROR

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.StdHepSplit'

# pylint: disable=protected-access


class StdHepSplitTestCase(unittest.TestCase):
  """Contains tests for the StdHepSplit class."""

  def setUp(self):
    """set up the objects."""
    self.shs = StdHepSplit()

  def test_applicationspecificinputs_nooutputfile(self):
    assertDiracFailsWith(self.shs.applicationSpecificInputs(), 'no output file defined', self)

  def test_applicationspecificinputs(self):
    self.shs.OutputFile = 'something'
    self.shs.workflow_commons.update(
        {'IS_PROD': True, 'ProductionOutputData':
         'ignore_,me;/some/dir/outputfile_gen_123.stdio;more_empty_entries.pdf;/dir/ignore;'})
    self.shs.InputFile = []
    self.shs.InputData = ['/myfile/1/ignoreme.txt', '/other/input/file.stdhep', 'more/ignored/', '',
                           '/add/me/input.stdhep', '/myfile/1/ignoremenot.stdhep']
    self.shs.step_commons['listoutput'] = [14897, True, 'something', {}, 'delete_this']
    assertDiracSucceedsWith_equals(self.shs.applicationSpecificInputs(), 'Parameters resolved', self)
    assertEqualsImproved(self.shs.OutputFile, 'outputfile_gen_123.stdio', self)
    assertEqualsImproved(self.shs.listoutput, 14897, self)

  def test_applicationspecificinputs_getoutputfile(self):
    self.shs.OutputFile = 'something'
    self.shs.workflow_commons.update({'IS_PROD': True, 'PRODUCTION_ID': '1248', 'JOB_ID': 173})
    self.shs.InputFile = 'input.stdhep'
    self.shs.InputData = []
    self.shs.listoutput = 'my_test_list'
    self.shs.step_commons['listoutput'] = []
    with patch('%s.getProdFilename' % MODULE_NAME, new=Mock(return_value='myoutputfile.stdio')) as getfile_mock:
      assertDiracSucceedsWith_equals(self.shs.applicationSpecificInputs(), 'Parameters resolved', self)
      assertEqualsImproved(self.shs.OutputFile, 'myoutputfile.stdio', self)
      assertEqualsImproved(self.shs.listoutput, 'my_test_list', self)
      getfile_mock.assert_called_once_with('something', 1248, 173)

  def test_applicationspecificinputs_2(self):
    self.shs.OutputFile = 'something'
    self.shs.workflow_commons.update({'IS_PROD': True, 'PRODUCTION_ID': '1248', 'JOB_ID': 173})
    self.shs.InputFile = 'input.stdhep'
    self.shs.InputData = []
    self.shs.listoutput = 'my_test_list'
    with patch('%s.getProdFilename' % MODULE_NAME, new=Mock(return_value='myoutputfile.stdio')) as getfile_mock:
      assertDiracSucceedsWith_equals(self.shs.applicationSpecificInputs(), 'Parameters resolved', self)
      assertEqualsImproved(self.shs.OutputFile, 'myoutputfile.stdio', self)
      assertEqualsImproved(self.shs.listoutput, 'my_test_list', self)
      getfile_mock.assert_called_once_with('something', 1248, 173)

  def test_applicationspecificinputs_isprod_false(self):
    self.shs.OutputFile = 'something'
    self.shs.InputFile = 'input.stdhep'
    self.shs.InputData = []
    assertDiracSucceedsWith_equals(self.shs.applicationSpecificInputs(), 'Parameters resolved', self)
    assertEqualsImproved(self.shs.OutputFile, 'something', self)
    assertEqualsImproved(self.shs.listoutput, {}, self)

  def test_execute_no_platform(self):
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock()) as resolve_mock:
      assertDiracFailsWith(self.shs.execute(), 'no ilc platform selected', self)
      resolve_mock.assert_called_once_with()

  def test_execute_resolve_input_fails(self):
    self.shs.platform = 'TestPlatV1'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_ERROR('test_resolve_err'))):
      assertDiracFailsWith(self.shs.execute(), 'test_resolve_err', self)

  def test_execute_resolve_paths_fails(self):
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = 'something'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_ERROR())) as resolve_mock:
      assertDiracFailsWith(self.shs.execute(), 'missing stdhep file', self)
      resolve_mock.assert_called_once_with('something')

  def test_execute_noinput(self):
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = ''
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())):
      assertDiracSucceedsWith_equals(self.shs.execute(), 'No files found to process', self)

  def test_execute_getswfolder_fails(self):
    self.shs.applicationVersion = 12
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = 'something'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['runonstd_test_hep']))), \
         patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_ERROR('test_getsw_err_dir'))) as getsw_mock:
      assertDiracFailsWith(self.shs.execute(), 'test_getsw_err_dir', self)
      getsw_mock.assert_called_once_with('TestPlatV1', 'stdhepsplit', 12)

  def test_execute_no_applog(self):
    exists_dict = {'StdHepSplit_12_Run_4.tcl': True, 'mytest_applog.log': False}
    expected_script = """
#!/bin/sh

################################################################################
# Dynamically generated script by LCIOConcatenate module                       #
################################################################################

declare -x LD_LIBRARY_PATH=mySplitDir/test/lib:/mydir/ldlibs/library

mySplitDir/test//hepsplit --infile runonstd_test_hep --nw_per_file 16 --outpref /some/dir/mytestOutput_file

exit $?

"""
    self.shs.applicationVersion = 12
    self.shs.STEP_NUMBER = 4
    self.shs.nbEventsPerSlice = 16
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = 'something'
    self.shs.applicationLog = 'mytest_applog.log'
    open_mock = Mock()
    self.shs.OutputFile = '/some/dir/mytestOutput_file.stdhep'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['runonstd_test_hep']))), \
         patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('mySplitDir/test/'))), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/mydir/ldlibs/library')) as getlibs_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as file_mock, \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK((0, )))) as shell_mock:
      assertDiracFailsWith(self.shs.execute(), 'failed reading the log file', self)
      getlibs_mock.assert_called_once_with('TestPlatV1', 'stdhepsplit', 12)
      file_mock.assert_called_once_with('StdHepSplit_12_Run_4.tcl', 'w')
      open_mock.close.assert_called_once_with()
      open_mock.write.assert_called_once_with(expected_script)
      remove_mock.assert_called_once_with('StdHepSplit_12_Run_4.tcl')
      chmod_mock.assert_called_once_with('StdHepSplit_12_Run_4.tcl', 0o755)
      shell_mock.assert_called_once_with(0, '"./StdHepSplit_12_Run_4.tcl"',
                                          callbackFunction=self.shs.redirectLogOutput, bufferLimit=20971520)

  def test_execute_maxread(self):
    exists_dict = {'StdHepSplit_V3_Run_4.tcl': True, 'mytest_applog.log': False}
    expected_script = """
#!/bin/sh

################################################################################
# Dynamically generated script by LCIOConcatenate module                       #
################################################################################

declare -x LD_LIBRARY_PATH=mySplitDir/test/lib:/mydir/ldlibs/library

mySplitDir/test//hepsplit --infile runonstd_test_hep --nw_per_file 16 --outpref /some/dir/mytestOutput_file --maxread 112

exit $?

"""
    self.shs.applicationVersion = 'V3'
    self.shs.STEP_NUMBER = 4
    self.shs.nbEventsPerSlice = 16
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = 'something'
    self.shs.maxRead = 111
    self.shs.applicationLog = 'mytest_applog.log'
    open_mock = Mock()
    self.shs.OutputFile = '/some/dir/mytestOutput_file.stdhep'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['runonstd_test_hep']))), \
         patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('mySplitDir/test/'))), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/mydir/ldlibs/library')) as getlibs_mock, \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)) as file_mock, \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK((0, )))) as shell_mock:
      assertDiracFailsWith(self.shs.execute(), 'failed reading the log file', self)
      getlibs_mock.assert_called_once_with('TestPlatV1', 'stdhepsplit', 'V3')
      file_mock.assert_called_once_with('StdHepSplit_V3_Run_4.tcl', 'w')
      open_mock.close.assert_called_once_with()
      open_mock.write.assert_called_once_with(expected_script)
      remove_mock.assert_called_once_with('StdHepSplit_V3_Run_4.tcl')
      chmod_mock.assert_called_once_with('StdHepSplit_V3_Run_4.tcl', 0o755)
      shell_mock.assert_called_once_with(0, '"./StdHepSplit_V3_Run_4.tcl"',
                                          callbackFunction=self.shs.redirectLogOutput, bufferLimit=20971520)

  def test_execute(self):
    exists_dict = {'StdHepSplit_12_Run_4.tcl': False, 'mytest_applog.log': True}
    self.shs.applicationVersion = 12
    self.shs.STEP_NUMBER = 4
    self.shs.nbEventsPerSlice = 16
    self.shs.workflow_commons['ProductionOutputData'] = 'first_entry;/some/dir/mytestOutput_file/new_folder;a;;dontdeleteme'
    self.shs.platform = 'TestPlatV1'
    self.shs.InputFile = 'something'
    self.shs.applicationLog = 'mytest_applog.log'
    self.shs.listoutput = {'outputPath': 123, 'outputDataSE': 'myTestDataSE'}
    open_mock = Mock()
    open_mock.__enter__.return_value = ['Open output file opfile1  ', 'Record = 41298 ',
                                         'Open output file protonpeter', 'Record = 48',
                                         'Record = 2 Output Begin Run',
                                         'Open output file     /mydir/files/run1.stdhep', 'Record=172',
                                         'Open output file       ignored_file',
                                         'Open output file /last/file.stdhep', 'Record =2941              ']
    self.shs.OutputFile = '/some/dir/mytestOutput_file.stdhep'
    with patch('%s.StdHepSplit.resolveInputVariables' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.resolveIFpaths' % MODULE_NAME, new=Mock(return_value=S_OK(['runonstd_test_hep']))), \
         patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_OK('mySplitDir/test/'))), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='/mydir/ldlibs/library')), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock()) as remove_mock, \
         patch('%s.open' % MODULE_NAME, new=Mock(return_value=open_mock)), \
         patch('%s.os.chmod' % MODULE_NAME, new=Mock()), \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK((0, )))), \
         patch('%s.StdHepSplit.listDir' % MODULE_NAME, new=Mock()):
      assertDiracSucceeds(self.shs.execute(), self)
      remove_mock.assert_called_once_with('mytest_applog.log')
      assertListContentEquals(self.shs.step_commons['listoutput'], [
          {'outputFile': 'opfile1', 'outputPath': 123, 'outputDataSE': 'myTestDataSE'},
          {'outputFile': 'protonpeter', 'outputPath': 123, 'outputDataSE': 'myTestDataSE'},
          {'outputFile': '/mydir/files/run1.stdhep', 'outputPath': 123, 'outputDataSE': 'myTestDataSE'},
          {'outputFile': 'ignored_file', 'outputPath': 123, 'outputDataSE': 'myTestDataSE'},
          {'outputFile': '/last/file.stdhep', 'outputPath': 123, 'outputDataSE': 'myTestDataSE'}], self)
      assertListContentEquals(
          self.shs.workflow_commons['ProductionOutputData'].split(';'),
          'first_entry;a;;dontdeleteme;/some/dir/mytestOutput_file/ignored_file;/some/dir/mytestOutput_file/opfile1;/some/dir/mytestOutput_file/protonpeter;/last/file.stdhep;/mydir/files/run1.stdhep'.split(';'),
          self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Created on Feb 10, 2012.

:author: Stephane Poss
"""

from __future__ import print_function
from __future__ import absolute_import
from DIRAC.Core.Base import Script

import sys
from six.moves import input

Script.registerSwitch('D:', 'description=', 'Short description of the workflow (default set by metadata)')
Script.registerSwitch('e:', 'evttype=', 'Name of the production event type (optional in addition to production ID)')
Script.registerSwitch('E:', 'energy=', 'Energy of the production events (default 3tev)')
Script.registerSwitch('m:', 'model=', 'Name of detector model to use')
Script.registerSwitch('n', 'nocheck', 'Switches off additional check before submission')
Script.registerSwitch('g:', 'group=', 'Name of the production group (default set by metadata)')
Script.registerSwitch('p:', 'prodid=', 'Production ID of input')
Script.registerSwitch('t:', 'time=', 'CPU time limit per job in seconds (default 300000)')
Script.registerSwitch('v:', 'version=', 'SLIC version to use (default v2r8p4)')
Script.registerSwitch('l:', 'lcsim=', 'LCSIM version to use (default 1.15-SNAPHOT)')
Script.registerSwitch('P:', 'pandora=', 'SlicPandora version to use (default CDR0)')

Script.registerSwitch('w:', 'workflow=', 'Name of the workflow (default set by metadata)')

Script.setUsageMessage(sys.argv[0] + ' -m <detectorModel> -p <prodID> (<additional options>)')

Script.parseCommandLine()
switches = Script.getUnprocessedSwitches()

detectorName = None
prodID = None
eventType = None
dataType = ''
energy = ''
prodGroup = None
cpuLimit = 300000
workflowName = None
workflowDescription = None
checkMeta = True

slicVersion = 'v2r8p4'
lcsimVers = '1.15-SNAPSHOT'
pandoraVers = 'CDR0'

slicMacro = 'defaultClicCrossingAngle.mac'
lcsimSteering1 = "clic_cdr_prePandora.lcsim"
lcsimSteering1_ov = "clic_cdr_prePandoraOverlay.lcsim"
lcsimSteering2 = "clic_cdr_postPandoraOverlay.lcsim"
pandoraSettings = "PandoraSettingsSlic.xml"
strategies = "defaultStrategies_%s.xml"


for switch in switches:
  opt = switch[0]
  arg = switch[1]
  if opt in ('D', 'description'):
    workflowDescription = arg
  elif opt in ('e', 'evttype'):
    eventType = arg
  elif opt in ('m', 'model'):
    detectorName = arg
    strategies = strategies % detectorName
  elif opt in ('g', 'group'):
    prodGroup = arg
  elif opt in ('p', 'prodid'):
    prodID = arg
  elif opt in ('t', 'time'):
    cpuLimit = arg
  elif opt in ('v', 'version'):
    slicVersion = arg
  elif opt in ('w', 'workflow'):
    workflowName = arg
  elif opt in ('n', 'nocheck'):
    checkMeta = False

if (detectorName is None) or (prodID is None):
  Script.showHelp()
  sys.exit(2)


from ILCDIRAC.Interfaces.API.NewInterface.ProductionJob import ProductionJob
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLIC, LCSIM, SLICPandora, OverlayInput
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
dirac = DiracILC()


meta = {}
meta['ProdID'] = prodID
if eventType:
  meta['EvtType'] = eventType
if energy:
  meta['Energy'] = energy
if dataType:
  meta['Datatype'] = dataType


fc = FileCatalogClient()
res = fc.getCompatibleMetadata(meta)
if not res['OK']:
  print("Error looking up the catalog for metadata")
  exit(2)

metaValues = res['Value']
#print metaValues

print('Meta data for production', prodID, ':')
for key, value in metaValues.items():
  if not len(value) == 0:
    print('\t', key, ':', value[0])

if dataType:
  print('\t', 'Datatype :', dataType)

if not eventType:
  eventType = metaValues['EvtType'][0]
else:
  print('\t', 'EvtType :', eventType)

if not energy:
  energy = metaValues['Energy'][0]
else:
  print('\t', 'Energy :', energy)

print('SLIC version:', slicVersion)
print('LCSim version:', lcsimVers)
print('SLICPandora version:', pandoraVers)
print('CPU limit:', cpuLimit, 'sec')


if not prodGroup:
  prodGroup = eventType + '_' + energy + '_cdr'
print('Production group:', prodGroup)

if not workflowName:
  workflowName = eventType + '_' + energy
print('Workflow name:', workflowName)

if not workflowDescription:
  workflowDescription = eventType + ' at ' + energy
print('Workflow description:', workflowDescription)


if checkMeta:
  answer = input('Submit production? (Y/N): ')
  if not answer.lower() in ('y', 'yes'):
    sys.exit(2)

# Define the applications

# Simulation
slic = SLIC()
slic.setVersion(slicVersion)
slic.setSteeringFile(slicMacro)
slic.setDetectorModel(detectorName)

# Reco without overlay: Do not take the input from SLIC as those are ran in different prods
lcsim_prepandora = LCSIM()
lcsim_prepandora.setVersion(lcsimVers)
lcsim_prepandora.setSteeringFile(lcsimSteering1)
lcsim_prepandora.setTrackingStrategy(strategies)
lcsim_prepandora.setDetectorModel(detectorName)
lcsim_prepandora.setOutputFile("prePandora.slcio")

slicpandora = SLICPandora()
slicpandora.setVersion(pandoraVers)
slicpandora.setDetectorModel(detectorName)
slicpandora.setPandoraSettings(pandoraSettings)
slicpandora.getInputFromApp(lcsim_prepandora)
slicpandora.setOutputFile('pandora.slcio')

lcsim_postpandora = LCSIM()
lcsim_postpandora.setVersion(lcsimVers)
lcsim_postpandora.getInputFromApp(slicpandora)
lcsim_postpandora.setSteeringFile(lcsimSteering2)
lcsim_postpandora.setTrackingStrategy(strategies)
lcsim_postpandora.setDetectorModel(detectorName)

# Now with overlay
overlay = OverlayInput()
overlay.setBXOverlay(60)
overlay.setGGToHadInt(1.3)  # When running at 1.4TeV
overlay.setDetectorModel("CLIC_SID_CDR")
overlay.setBkgEvtType("gghad")

lcsim_prepandora_ov = LCSIM()
lcsim_prepandora_ov.setVersion(lcsimVers)
lcsim_prepandora_ov.setSteeringFile(lcsimSteering1_ov)
lcsim_prepandora_ov.setTrackingStrategy(strategies)
lcsim_prepandora_ov.setDetectorModel(detectorName)
lcsim_prepandora_ov.setOutputFile("prePandora.slcio")

slicpandora_ov = SLICPandora()
slicpandora_ov.setVersion(pandoraVers)
slicpandora_ov.setDetectorModel(detectorName)
slicpandora_ov.setPandoraSettings(pandoraSettings)
slicpandora_ov.getInputFromApp(lcsim_prepandora_ov)
slicpandora_ov.setOutputFile('pandora.slcio')

lcsim_postpandora_ov = LCSIM()
lcsim_postpandora_ov.setVersion(lcsimVers)
lcsim_postpandora_ov.getInputFromApp(slicpandora_ov)
lcsim_postpandora_ov.setSteeringFile(lcsimSteering2)
lcsim_postpandora_ov.setTrackingStrategy(strategies)
lcsim_postpandora_ov.setDetectorModel(detectorName)

# 33
# Now define the productions

pslic = ProductionJob()
pslic.setInputDataQuery(meta)
res = pslic.append(slic)
if not res['OK']:
  print(res['Message'])
  exit(1)
pslic.addFinalization(True, True, True, True)

pslic.setProdType("MCSimulation")
pslic.setProdGroup(prodGroup)
pslic.setWorkflowName(workflowName + '_sim_sid_cdr')
pslic.setCPUTime(cpuLimit)
pslic.setOutputSE("CERN-SRM")
pslic.setDescription('Simulating ' + workflowDescription)
res = pslic.createProduction()
if not res['OK']:
  print(res['Message'])
  exit(1)
res = pslic.finalizeProd()
if not res['OK']:
  print(res['Message'])
  exit(1)

meta = pslic.getMetadata()  # This is needed to link Sim and Rec


# Reconstruction w/o overlay
plcsim = ProductionJob()
plcsim.setInputDataQuery(meta)
res = plcsim.append(lcsim_prepandora)
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim.append(slicpandora)
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim.append(lcsim_postpandora)
if not res['OK']:
  print(res['Message'])
  exit(1)
plcsim.addFinalization(True, True, True, True)

plcsim.setProdType("MCReconstruction")
plcsim.setProdGroup(prodGroup)
plcsim.setWorkflowName(workflowName + '_rec_sid_cdr')
plcsim.setCPUTime(cpuLimit)
plcsim.setOutputSE("CERN-SRM")
plcsim.setDescription('Reconstructing ' + workflowDescription)
res = plcsim.createProduction()
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim.finalizeProd()
if not res['OK']:
  print(res['Message'])
  exit(1)


# Now do the reconstruction with overlay
plcsim_ov = ProductionJob()
plcsim_ov.setInputDataQuery(meta)

res = plcsim_ov.append(overlay)
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim_ov.append(lcsim_prepandora_ov)
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim_ov.append(slicpandora_ov)
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim_ov.append(lcsim_postpandora_ov)
if not res['OK']:
  print(res['Message'])
  exit(1)
plcsim_ov.addFinalization(True, True, True, True)

plcsim_ov.setProdType("MCReconstruction_Overlay")
plcsim_ov.setProdGroup(prodGroup)
plcsim_ov.setWorkflowName(workflowName + '_rec_sid_cdr_overlay')
plcsim_ov.setCPUTime(cpuLimit)
plcsim_ov.setOutputSE("CERN-SRM")
plcsim_ov.setDescription('Reconstructing with overlay ' + workflowDescription)
res = plcsim_ov.createProduction()
if not res['OK']:
  print(res['Message'])
  exit(1)
res = plcsim_ov.finalizeProd()
if not res['OK']:
  print(res['Message'])
  exit(1)
